<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Data Access Object for `challenge_invite` table.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_plugins.challenge.bol
 * @since 1.0
 */
class CHALLENGE_BOL_ChallengeInviteDao extends OW_BaseDao
{
    const USER_ID = 'userId';
    const INVITER_ID = 'inviterId';
    const TIME_STAMP = 'timeStamp';
    const CHALLENGE_ID = 'challengeId';

    /**
     * Singleton instance.
     *
     * @var CHALLENGE_BOL_ChallengeInviteDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return CHALLENGE_BOL_ChallengeInviteDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    protected function __construct()
    {
        parent::__construct();
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'CHALLENGE_BOL_ChallengeInvite';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'challenge_invite';
    }

    /**
     * @param integer $challengeId
     * @param integer $userId
     * @return CHALLENGE_BOL_ChallengeInvite
     */
    public function findObjectByUserIdAndChallengeId( $challengeId, $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::CHALLENGE_ID, (int) $challengeId);
        $example->andFieldEqual(self::USER_ID, (int) $userId);

        return $this->findObjectByExample($example);
    }

    /**
     * @param integer $challengeId
     * @param integer $userId
     */
    public function hideInvitationByUserId( $userId )
    {
        $query = "UPDATE `" . CHALLENGE_BOL_ChallengeInviteDao::getInstance()->getTableName() . "` SET `displayInvitation` = false 
            WHERE `" . CHALLENGE_BOL_ChallengeInviteDao::USER_ID . "` = :userId AND `displayInvitation` = true ";

        return $this->dbo->update($query, array('userId' => (int) $userId));
    }

    /**
     * @param integer $challengeId
     */
    public function deleteByChallengeId( $challengeId )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::CHALLENGE_ID, (int) $challengeId);

        $this->deleteByExample($example);
    }

    /**
     * @param integer $challengeId
     * @param integer $userId
     */
    public function deleteByUserIdAndChallengeId( $challengeId, $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::CHALLENGE_ID, (int)$challengeId);
        $example->andFieldEqual(self::USER_ID, (int)$userId);

        $this->deleteByExample($example);
    }

    /**
     * @param integer $challengeId
     */
    public function findInviteListByChallengeId( $challengeId)
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::CHALLENGE_ID, (int)$challengeId);

        return $this->findListByExample($example);
    }

    /**
     * @param integer $challengeId
     */
    public function findUserListForInvite( $challengeId, $first, $count, $friendList = null )
    {

        $userDao = BOL_UserDao::getInstance();
        $challengeDao = CHALLENGE_BOL_ChallengeDao::getInstance();
        $challengeUserDao = CHALLENGE_BOL_ChallengeUserDao::getInstance();

        $where = "";
        if ( isset($friendList) && empty($friendList) )
        {
            return array();
        }
        else if ( !empty($friendList) )
        {
            $where = " AND `u`.id IN ( " . $this->dbo->mergeInClause($friendList) . " ) ";
        }

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("u", "id", array(
            "method" => "CHALLENGE_BOL_ChallengeUserDao::findUserListForInvite"
        ));

        $query = "SELECT `u`.`id`
    		FROM `{$userDao->getTableName()}` as `u`
            " . $queryParts['join'] . "
                
            LEFT JOIN `" . $challengeDao->getTableName() . "` as `e`
    			ON( `u`.`id` = `e`.`userId` AND e.id = :challenge )
                
            LEFT JOIN `" . $this->getTableName() . "` as `ei`
    			ON( `u`.`id` = `ei`.`userId` AND `ei`.challengeId = :challenge )

            LEFT JOIN `" . $challengeUserDao->getTableName() . "` as `eu`
    			ON( `u`.`id` = `eu`.`userId` AND `eu`.challengeId = :challenge )

    		WHERE  " . $queryParts['where'] . " AND `e`.`id` IS NULL AND `ei`.`id` IS NULL AND `eu`.`id` IS NULL ". $where ."
    		ORDER BY `u`.`activityStamp` DESC
    		LIMIT :first, :count ";

        return $this->dbo->queryForColumnList($query, array('challenge' => $challengeId, 'first' => $first, 'count' => $count));
    }
}
