<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  Challenges Service.
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>, Podyachev Evgeny <joker.OW2@gmail.com>
 * @package ow_plugins.challenge.bol
 * @since 1.0
 */
final class CHALLENGE_BOL_ChallengeService
{
    const USER_STATUS_YES = CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_YES;
    const USER_STATUS_MAYBE = CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_MAYBE;
    const USER_STATUS_NO = CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_NO;

    const CAN_INVITE_PARTICIPANT = CHALLENGE_BOL_ChallengeDao::VALUE_WHO_CAN_INVITE_PARTICIPANT;
    const CAN_INVITE_CREATOR = CHALLENGE_BOL_ChallengeDao::VALUE_WHO_CAN_INVITE_CREATOR;

    const CAN_VIEW_ANYBODY = CHALLENGE_BOL_ChallengeDao::VALUE_WHO_CAN_VIEW_ANYBODY;
    const CAN_VIEW_INVITATION_ONLY = CHALLENGE_BOL_ChallengeDao::VALUE_WHO_CAN_VIEW_INVITATION_ONLY;
	
	const COMPLETADO = CHALLENGE_BOL_ChallengeDao::VALUE_STATUS_COMPLETADO;
	const CURSO = CHALLENGE_BOL_ChallengeDao::VALUE_STATUS_CURSO;
	const FALLIDO = CHALLENGE_BOL_ChallengeDao::VALUE_STATUS_FALLIDO;

    const CONF_CHALLENGE_USERS_COUNT = 'challenge_users_count';
    const CONF_CHALLENGE_USERS_COUNT_ON_PAGE = 'challenge_users_count_on_page';
    const CONF_CHALLENGES_COUNT_ON_PAGE = 'challenges_count_on_page';
    const CONF_WIDGET_CHALLENGES_COUNT = 'challenges_widget_count';
    const CONF_WIDGET_CHALLENGES_COUNT_OPTION_LIST = 'challenges_widget_count_select_set';
    const CONF_DASH_WIDGET_CHALLENGES_COUNT = 'challenges_dash_widget_count';

    const CHALLENGE_AFTER_CHALLENGE_EDIT = 'challenge_after_challenge_edit';
    const CHALLENGE_ON_DELETE_CHALLENGE = 'challenge_on_delete_challenge';
    const CHALLENGE_ON_CREATE_CHALLENGE = 'challenge_on_create_challenge';
    const CHALLENGE_ON_CHANGE_USER_STATUS = 'challenge_on_change_user_status';
    const CHALLENGE_AFTER_CREATE_CHALLENGE = 'challenge_after_create_challenge';
    
    const CHALLENGE_BEFORE_CHALLENGE_CREATE = 'challenges.before_challenge_create';
    const CHALLENGE_BEFORE_CHALLENGE_EDIT = 'challenges.before_challenge_edit';
    const CHALLENGE_COLLECT_TOOLBAR = 'challenges.collect_toolbar';

    /**
     * @var array
     */
    private $configs = array();
    /**
     * @var CHALLENGE_BOL_ChallengeDao
     */
    private $challengeDao;
    /**
     * @var CHALLENGE_BOL_ChallengeUserDao
     */
    private $challengeUserDao;
    /**
     * @var CHALLENGE_BOL_ChallengeInviteDao
     */
    private $challengeInviteDao;
    /**
     * Singleton instance.
     *
     * @var CHALLENGE_BOL_ChallengeService
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return CHALLENGE_BOL_ChallengeService
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    private function __construct()
    {
        $this->challengeDao = CHALLENGE_BOL_ChallengeDao::getInstance();
        $this->challengeUserDao = CHALLENGE_BOL_ChallengeUserDao::getInstance();
        $this->challengeInviteDao = CHALLENGE_BOL_ChallengeInviteDao::getInstance();

        $this->configs[self::CONF_CHALLENGE_USERS_COUNT] = 10;
        $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE] = 15;
        $this->configs[self::CONF_DASH_WIDGET_CHALLENGES_COUNT] = 3;
        $this->configs[self::CONF_WIDGET_CHALLENGES_COUNT] = 3;
        $this->configs[self::CONF_CHALLENGE_USERS_COUNT_ON_PAGE] = 30;
        $this->configs[self::CONF_WIDGET_CHALLENGES_COUNT_OPTION_LIST] = array(3 => 3, 5 => 5, 10 => 10, 15 => 15, 20 => 20);
    }

    /**
     * @return array
     */
    public function getConfigs()
    {
        return $this->configs;
    }

    /**
     * Saves challenge dto.
     *
     * @param CHALLENGE_BOL_Challenge $challenge
     */
    public function saveChallenge( CHALLENGE_BOL_Challenge $challenge )
    {
        $this->challengeDao->save($challenge);
    }

    /**
     * Makes and saves challenge standard image and icon.
     *
     * @param string $imagePath
     * @param integer $imageId
     */
    public function saveChallengeImage( $imagePath, $imageId )
    {
        $storage = OW::getStorage();
        
        if ( $storage->fileExists($this->generateImagePath($imageId)) )
        {
            $storage->removeFile($this->generateImagePath($imageId));
            $storage->removeFile($this->generateImagePath($imageId, false));
        }

        $pluginfilesDir = Ow::getPluginManager()->getPlugin('challenge')->getPluginFilesDir();

        $tmpImgPath = $pluginfilesDir . 'img_' .uniqid() . '.jpg';
        $tmpIconPath = $pluginfilesDir . 'icon_' . uniqid() . '.jpg';

        $image = new UTIL_Image($imagePath);
        $image->resizeImage(400, null)->saveImage($tmpImgPath)
            ->resizeImage(100, 100, true)->saveImage($tmpIconPath);

        unlink($imagePath);

        $storage->copyFile($tmpIconPath, $this->generateImagePath($imageId));
        $storage->copyFile($tmpImgPath,$this->generateImagePath($imageId, false));

        unlink($tmpImgPath);
        unlink($tmpIconPath);
    }

    /**
     * Deletes challenge.
     *
     * @param integer $challengeId
     */
    public function deleteChallenge( $challengeId )
    {
        $challengeDto = $this->challengeDao->findById((int) $challengeId);

        if ( $challengeDto === null )
        {
            return;
        }
        
        $e = new OW_Challenge(self::CHALLENGE_ON_DELETE_CHALLENGE, array('challengeId' => (int) $challengeId));
        OW::getChallengeManager()->trigger($e);

        if( !empty($challengeDto->image) )
        {
            $storage = OW::getStorage();
            $storage->removeFile($this->generateImagePath($challengeDto->image));
            $storage->removeFile($this->generateImagePath($challengeDto->image, false));
        }

        $this->challengeUserDao->deleteByChallengeId($challengeDto->getId());
        $this->challengeDao->deleteById($challengeDto->getId());
        $this->challengeInviteDao->deleteByChallengeId($challengeDto->getId());
        BOL_InvitationService::getInstance()->deleteInvitationByEntity('challenge', $challengeId);
        BOL_InvitationService::getInstance()->deleteInvitationByEntity('challenge-invitation', $challengeId);
    }

    /**
     * Returns challenge image and icon path.
     *
     * @param integer $imageId
     * @param boolean $icon
     * @return string
     */
    public function generateImagePath( $imageId, $icon = true )
    {
        $imagesDir = OW::getPluginManager()->getPlugin('challenge')->getUserFilesDir();
        return $imagesDir . ( $icon ? 'challenge_icon_' : 'challenge_image_' ) . $imageId . '.jpg';
    }

    /**
     * Returns challenge image and icon url.
     * 
     * @param integer $imageId
     * @param boolean $icon
     * @return string
     */
    public function generateImageUrl( $imageId, $icon = true )
    {
        return OW::getStorage()->getFileUrl($this->generateImagePath($imageId, $icon));
    }

    /**
     * Returns default challenge image url.
     */
    public function generateDefaultImageUrl()
    {
        return OW::getThemeManager()->getCurrentTheme()->getStaticImagesUrl() . 'no-picture.png';
    }

    /**
     * Finds challenge by id.
     *
     * @param integer $id
     * @return CHALLENGE_BOL_Challenge
     */
    public function findChallenge( $id )
    {
        return $this->challengeDao->findById((int) $id);
    }

    /**
     * Returns challenge users with provided status.
     *
     * @param integer $challengeId
     * @param integer $status
     * @return array<CHALLENGE_BOL_ChallengeUser>
     */
    public function findChallengeUsers( $challengeId, $status, $page, $usersCount = null )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $usersCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGE_USERS_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeUserDao->findListByChallengeIdAndStatus($challengeId, $status, $first, $count);
    }

    /**
     * Returns users count for provided challenge and status.
     *
     * @param integer $challengeId
     * @param integer $status
     * @return integer
     */
    public function findChallengeUsersCount( $challengeId, $status )
    {
        return (int) $this->challengeUserDao->findUsersCountByChallengeIdAndStatus($challengeId, $status);
    }

    /**
     * Saves challenge user objects.
     *
     * @param CHALLENGE_BOL_ChallengeUser $challengeUser
     */
    public function saveChallengeUser( CHALLENGE_BOL_ChallengeUser $challengeUser )
    {
        $this->challengeUserDao->save($challengeUser);
    }

    /**
     * Saves challenge user objects.
     *
     * @param CHALLENGE_BOL_ChallengeUser $challengeUser
     */
    public function addChallengeUser( $userId, $challengeId, $status, $timestamp = null )
    {
        $statusList = array( CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_YES, CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_MAYBE, CHALLENGE_BOL_ChallengeUserDao::VALUE_STATUS_NO );

        if( (int) $userId <= 0 || $challengeId <=0 || !in_array($status, $statusList) )
        {
            return null;
        }

        $challenge = $this->findChallenge($challengeId);

        if( empty($challenge) )
        {
            return null;
        }

        if ( !isset($timestamp) )
        {
            $timestamp = time();
        }

        $challengeUser = $this->findChallengeUser($challengeId, $userId);

        if ( empty($challengeUser) )
        {
            $challengeUser = new CHALLENGE_BOL_ChallengeUser();

            $challengeUser->challengeId = $challengeId;
            $challengeUser->userId = $userId;
            $challengeUser->timeStamp = $timestamp;
        }

        $challengeUser->status = $status;
        
        $this->challengeUserDao->save($challengeUser);

        return $challengeUser;
    }

    /**
     * Finds challenge-user object.
     *
     * @param integer $challengeId
     * @param integer $userId
     * @return CHALLENGE_BOL_ChallengeUser
     */
    public function findChallengeUser( $challengeId, $userId )
    {
        return $this->challengeUserDao->findObjectByChallengeIdAndUserId($challengeId, $userId);
    }

    /**
     * Checks if user can view and join challenge.
     *
     * @param integer $challengeId
     * @param integer $userId
     * @return boolean
     */
    public function canUserView( $challengeId, $userId )
    {
        $challenge = $this->challengeDao->findById($challengeId);
        /* @var $challenge CHALLENGE_BOL_Challenge */
        if ( $challenge === null )
        {
            return false;
        }

        $userChallenge = $this->challengeUserDao->findObjectByChallengeIdAndUserId($challengeId, $userId);

        if ( $challenge->getWhoCanView() === self::CAN_VIEW_INVITATION_ONLY && $userChallenge === null )
        {
            return false;
        }

        return true;
    }

    /**
     * Checks if user can invite to challenge.
     *
     * @param integer $challengeId
     * @param integer $userId
     * @return boolean
     */
    public function canUserInvite( $challengeId, $userId )
    {
        $challenge = $this->challengeDao->findById($challengeId);
        /* @var $challenge CHALLENGE_BOL_Challenge */
        if ( $challenge === null || ( $challenge->getWhoCanInvite() == self::CAN_INVITE_CREATOR && $userId != $challenge->getUserId() ) )
        {
            return false;
        }

        $userChallenge = $this->challengeUserDao->findObjectByChallengeIdAndUserId($challengeId, $userId);

        if ( $userChallenge === null || $userChallenge->getStatus() != self::USER_STATUS_YES )
        {
            return false;
        }

        return true;
    }

    /**
     * Returns latest challenges list.
     *
     * @param integer $page
     * @return array<CHALLENGE_BOL_Challenge>
     */
    public function findPublicChallenges( $page, $challengesCount = null, $past = false )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $challengesCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeDao->findPublicChallenges($first, $count, $past);
    }

    /**
     * Returns latest challenges count.
     *
     * @return integer
     */
    public function findPublicChallengesCount( $past = false )
    {
        return $this->challengeDao->findPublicChallengesCount($past);
    }

    /**
     * Invites user to challenge.
     *
     * @param integer $challengeId
     * @param integer $userId
     * @param integer $inviterId
     *
     * @return CHALLENGE_BOL_ChallengeInvite
     */
    public function inviteUser( $challengeId, $userId, $inviterId )
    {
        $challenge = $this->findChallenge($challengeId);

        if ( $challenge === null )
        {
            return false;
        }

        $challengeInvite = new CHALLENGE_BOL_ChallengeInvite();
        $challengeInvite->setChallengeId($challengeId);
        $challengeInvite->setInviterId($inviterId);
        $challengeInvite->setUserId($userId);
        $challengeInvite->setTimeStamp(time());
        $challengeInvite->setDisplayInvitation(true);

        $this->challengeInviteDao->save($challengeInvite);

        return $challengeInvite;
    }

    /**
     * Returns challenge invitation for user.
     *
     * @param integer $challengeId
     * @param integer $userId
     * @return CHALLENGE_BOL_ChallengeInvite
     */
    public function findChallengeInvite( $challengeId, $userId )
    {
        return $this->challengeInviteDao->findObjectByUserIdAndChallengeId($challengeId, $userId);
    }

    /**
     * Finds challenges for user
     *
     * @param integer $userId
     * @return array
     */
    public function findUserChallenges( $userId, $page, $challengesCount = null )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $challengesCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeDao->findUserCreatedChallenges($userId, $first, $count);
    }

    /**
     * Returns user created challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function findUsersChallengesCount( $userId )
    {
        return $this->challengeDao->findUserCretedChallengesCount($userId);
    }

    /**
     * Returns list of user participating challenges.
     *
     * @param integer $userId
     * @param integer $page
     * @param integer $count
     * @return array
     */
    public function findUserParticipatedChallenges( $userId, $page, $challengesCount = null )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $challengesCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeDao->findUserChallengesWithStatus($userId, self::USER_STATUS_YES, $first, $count);
    }

    /**
     * Returns user participated challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function findUserParticipatedChallengesCount( $userId )
    {
        return $this->challengeDao->findUserChallengesCountWithStatus($userId, self::USER_STATUS_YES);
    }

    /**
     * Returns list of user participating public challenges.
     *
     * @param integer $userId
     * @param integer $page
     * @param integer $count
     * @return array
     */
    public function findUserParticipatedPublicChallenges( $userId, $page, $challengesCount = null )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $challengesCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeDao->findPublicUserChallengesWithStatus($userId, self::USER_STATUS_YES, $first, $count);
    }

    /**
     * Returns user participated public challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function findUserParticipatedPublicChallengesCount( $userId )
    {
        return $this->challengeDao->findPublicUserChallengesCountWithStatus($userId, self::USER_STATUS_YES);
    }

    /**
     * Returns user participated public challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function hideInvitationByUserId( $userId )
    {
        return $this->challengeInviteDao->hideInvitationByUserId($userId);
    }

    /**
     * Prepares data for ipc listing.
     *
     * @param array<CHALLENGE_BOL_Challenge> $challenges
     * @return array
     */
    public function getListingData( array $challenges )
    {
        $resultArray = array();

        /* @var $challengeItem CHALLENGE_BOL_Challenge */
        foreach ( $challenges as $challengeItem )
        {
            $title = UTIL_String::truncate(strip_tags($challengeItem->getTitle()), 80, "...") ;
            $content = UTIL_String::truncate(strip_tags($challengeItem->getDescription()), 100, "...");
            $status=$challengeItem->getStatus();
			
            $resultArray[$challengeItem->getId()] = array(
                'content' => $content,
                'title' => $title,
                'challengeUrl' => OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challengeItem->getId())),
                'imageSrc' => ( $challengeItem->getImage() ? $this->generateImageUrl($challengeItem->getImage(), true) : $this->generateDefaultImageUrl() ),
                'imageTitle' => $title,
				'status'=>$status
            );
        }

        return $resultArray;
    }

    /**
     * Prepares data for ipc listing with toolbar.
     *
     * @param array<CHALLENGE_BOL_Challenge> $challenges
     * @return array
     */
    public function getListingDataWithToolbar( array $challenges, $toolbarList = array() )
    {
        $resultArray = $this->getListingData($challenges);
        $userService = BOL_UserService::getInstance();

        $idArray = array();

        /* @var $challenge CHALLENGE_BOL_Challenge */
        foreach ( $challenges as $challenge )
        {
            $idArray[] = $challenge->getUserId();
        }

        $usernames = $userService->getDisplayNamesForList($idArray);
        $urls = $userService->getUserUrlsForList($idArray);

        $language = OW::getLanguage();
        /* @var $challengeItem CHALLENGE_BOL_Challenge */
        foreach ( $challenges as $challengeItem )
        {
            $resultArray[$challengeItem->getId()]['toolbar'][] = array('label' => $usernames[$challengeItem->getUserId()], 'href' => $urls[$challengeItem->getUserId()], 'class' => 'ow_icon_control ow_ic_user');
            $resultArray[$challengeItem->getId()]['toolbar'][] = array('label' => UTIL_DateTime::formatSimpleDate($challengeItem->getStartTimeStamp(),$challengeItem->getStartTimeDisable()), 'class' => 'ow_ipc_date');

            if ( !empty($toolbarList[$challengeItem->getId()]) )
            {
                $resultArray[$challengeItem->getId()]['toolbar'] = array_merge($resultArray[$challengeItem->getId()]['toolbar'], $toolbarList[$challengeItem->getId()]);
            }
            
            /* if( !empty($isInviteList) )
            {
                $resultArray[$challengeItem->getId()]['toolbar'][] = array('label' => $language->text('challenge', 'ignore_request'),'href' => 'challenge.invite_acept');
                $resultArray[$challengeItem->getId()]['toolbar'][] = array('label' => $language->text('challenge', 'accept_request'),'href' => 'challenge.invite_decline');
            }*/
        }
        //printVar($resultArray);
        return $resultArray;
    }

    public function getUserListsArray()
    {
        return array(
            self::USER_STATUS_YES => 'yes',
            self::USER_STATUS_MAYBE => 'maybe',
            self::USER_STATUS_NO => 'no'
        );
    }

    /**
     * Returns user invited challenges.
     *
     * @param integer $userId
     * @param integer $page
     * @param integer $challengesCount
     * @return array<CHALLENGE_BOL_Challenge>
     */
    public function findUserInvitedChallenges( $userId, $page, $challengesCount = null )
    {
        if ( $page === null )
        {
            $first = 0;
            $count = (int) $challengesCount;
        }
        else
        {
            $page = ( $page === null ) ? 1 : (int) $page;
            $count = $this->configs[self::CONF_CHALLENGES_COUNT_ON_PAGE];
            $first = ( $page - 1 ) * $count;
        }

        return $this->challengeDao->findUserInvitedChallenges($userId, $first, $count);
    }

    /**
     * Returns user invited challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function findUserInvitedChallengesCount( $userId )
    {
        return $this->challengeDao->findUserInvitedChallengesCount($userId);
    }

    /**
     * Returns displayed user invited challenges count.
     *
     * @param integer $userId
     * @return integer
     */
    public function findDisplayedUserInvitationCount( $userId )
    {
        return $this->challengeDao->findDisplayedUserInvitationCount($userId);
    }

    /**
     * Deletes all challenge invites for provided user.
     *
     * @param integer $challengeId
     * @param integer $userId
     */
    public function deleteUserChallengeInvites( $challengeId, $userId )
    {
        $this->challengeInviteDao->deleteByUserIdAndChallengeId($challengeId, $userId);
    }

    /**
     * Deletes all user challenges.
     *
     * @param integer $userId
     */
    public function deleteUserChallenges( $userId )
    {
        $challenges = $this->challengeDao->findAllUserChallenges($userId);

        /* @var $challenge CHALLENGE_BOL_Challenge */
        foreach ( $challenges as $challenge )
        {
            $this->deleteChallenge($challenge->getId());
        }
    }

    /**
     * returns invited userId list.
     *
     * @param integer $challengeId
     */
    public function findInviteUserListByChallengeId( $challengeId )
    {
        $inviteList = $this->challengeInviteDao->findInviteListByChallengeId($challengeId);

        $userList = array();

        foreach ( $inviteList as $invite )
        {
            $userList[] = $invite->userId;
        }

        return $userList;
    }

    public function findUserListForInvite( $challengeId, $first, $count, $friendList = array() )
    {
         return $this->challengeInviteDao->findUserListForInvite($challengeId, $first, $count, $friendList );
    }
    
    public function getContentMenu()
    {
        $menuItems = array();

        if ( OW::getUser()->isAuthenticated() )
        {
            $listNames = array(
                'invited' => array('iconClass' => 'ow_ic_bookmark'),
                'joined' => array('iconClass' => 'ow_ic_friends'),
                'past' => array('iconClass' => 'ow_ic_reply'),
                'latest' => array('iconClass' => 'ow_ic_reply')
            );
        }
        else
        {
            $listNames = array(
                'past' => array('iconClass' => 'ow_ic_reply'),
                'latest' => array('iconClass' => 'ow_ic_reply')
            );
        }
        
        foreach ( $listNames as $listKey => $listArr )
        {
            $menuItem = new BASE_MenuItem();
            $menuItem->setKey($listKey);
            $menuItem->setUrl(OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => $listKey)));
            $menuItem->setLabel(OW::getLanguage()->text('challenge', 'common_list_type_' . $listKey . '_label'));
            $menuItem->setIconClass($listArr['iconClass']);
            $menuItems[] = $menuItem;
        }
        
        $challenge = new BASE_CLASS_ChallengeCollector('challenge.add_content_menu_item');
        OW::getChallengeManager()->getInstance()->trigger($challenge);
        
        $data = $challenge->getData();
        
        if ( !empty($data) && is_array($data) )
        {
            $menuItems = array_merge($menuItems, $data);
        }
        
        return new BASE_CMP_ContentMenu($menuItems);
    }
    
    public function findByIdList( $idList )
    {
        return $this->challengeDao->findByIdList($idList);
    }
}