<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class CHALLENGE_CLASS_ChallengeHandler
{

    public function __construct()
    {
        
    }

    public function onNotifyActions( BASE_CLASS_ChallengeCollector $e )
    {
        $e->add(array(
            'section' => 'challenge',
            'action' => 'challenge-invitation',
            'sectionIcon' => 'ow_ic_calendar',
            'sectionLabel' => OW::getLanguage()->text('challenge', 'notifications_section_label'),
            'description' => OW::getLanguage()->text('challenge', 'notifications_new_message'),
            'selected' => true
        ));

        $e->add(array(
            'section' => 'challenge',
            'sectionIcon' => 'ow_ic_files',
            'sectionLabel' => OW::getLanguage()->text('challenge', 'notifications_section_label'),
            'action' => 'challenge-add_comment',
            'description' => OW::getLanguage()->text('challenge', 'email_notification_comment_setting'),
            'selected' => true
        ));
    }

    public function onUserInvite( OW_Challenge $e )
    {
        $params = $e->getParams();

        OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeUserDao::CACHE_TAG_CHALLENGE_USER_LIST . $params['challengeId']));
    }

    /**
     * Add challenge entity to the newsfeed
     *
     * @param OW_Challenge $e
     */
    public function feedEntityAdd( OW_Challenge $e )
    {
        $params = $e->getParams();
        $data = $e->getData();

        if ( $params['entityType'] != 'challenge' )
        {
            return;
        }

        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();
        $challenge = $challengeService->findChallenge($params['entityId']);

        //    if ( $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY )
        //    {
        //        return;
        //    }
        //$url = OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->getId()));
        //$thumb = $challengeService->generateImageUrl($challenge->image, true);

        $title = UTIL_String::truncate(strip_tags($challenge->getTitle()), 100, "...");


        /* $data = array(
          'time' => $challenge->getCreateTimeStamp(),
          'ownerId' => $challenge->getUserId(),
          'string' => OW::getLanguage()->text('challenge', 'feed_add_item_label'),
          'content' => '<div class="clearfix"><div class="ow_newsfeed_item_picture">
          <a href="' . $url . '"><img src="' . ( $challenge->getImage() ? $challengeService->generateImageUrl($challenge->getImage(), true) : $challengeService->generateDefaultImageUrl() ) . '" /></a>
          </div><div class="ow_newsfeed_item_content">
          <a class="ow_newsfeed_item_title" href="' . $url . '">' . $title . '</a><div class="ow_remark ow_smallmargin">' . UTIL_String::truncate(strip_tags($challenge->getDescription()), 200, '...') . '</div><div class="ow_newsfeed_action_activity challenge_newsfeed_activity">[ph:activity]</div></div></div>',
          'view' => array(
          'iconClass' => 'ow_ic_calendar'
          )
          ); */

        $data = array(
            'time' => $challenge->getCreateTimeStamp(),
            'ownerId' => $challenge->getUserId(),
            'string' => OW::getLanguage()->text('challenge', 'feed_add_item_label'), //array("key" => "challenge+feed_add_item_label"),
            'content' => array(
                "format" => "image_content",
                "vars" => array(
                    "image" => ( $challenge->getImage() ? $challengeService->generateImageUrl($challenge->getImage(), false) : $challengeService->generateDefaultImageUrl() ),
                    "thumbnail" => ( $challenge->getImage() ? $challengeService->generateImageUrl($challenge->getImage(), true) : $challengeService->generateDefaultImageUrl() ),
                    "title" => $title,
                    "description" => UTIL_String::truncate(strip_tags($challenge->getDescription()), 200, '...'),
                    "url" => array(
                        "routeName" => 'challenge.view',
                        "vars" => array('challengeId' => $challenge->getId())
                    ),
                    'iconClass' => 'ow_ic_challenge'
                )
            ),
            'view' => array(
                'iconClass' => 'ow_ic_calendar'
            ),
        );

        if ( $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY )
        {
            $data['params']['visibility'] = 4;
        }

        $e->setData($data);
    }

    public function afterChallengeEdit( OW_Challenge $e )
    {
        $params = $e->getParams();
        $challengeId = (int) $params['challengeId'];

        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();
        $challenge = $challengeService->findChallenge($challengeId);

        //$url = OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->getId()));
        // $thumb = $challengeService->generateImageUrl($challenge->image, true);

        /* $data = array(
          'time' => $challenge->getCreateTimeStamp(),
          'ownerId' => $challenge->getUserId(),
          'string' => OW::getLanguage()->text('challenge', 'feed_add_item_label'),
          'content' => '<div class="clearfix"><div class="ow_newsfeed_item_picture">
          <a href="' . $url . '"><img src="' . $thumb . '" /></a>
          </div><div class="ow_newsfeed_item_content">
          <a class="ow_newsfeed_item_title" href="' . $url . '">' . $challenge->getTitle() . '</a><div class="ow_remark ow_smallmargin">' . UTIL_String::truncate(strip_tags($challenge->getDescription()), 200, '...') . '</div><div class="ow_newsfeed_action_activity challenge_newsfeed_activity">[ph:activity]</div></div></div>',
          'view' => array(
          'iconClass' => 'ow_ic_calendar'
          )
          ); */

        $data = array(
            'time' => $challenge->getCreateTimeStamp(),
            'ownerId' => $challenge->getUserId(),
            'string' => array("key" => "challenge+feed_add_item_label"), //OW::getLanguage()->text('challenge', 'feed_add_item_label'),
            'content' => array(
                "format" => "image_content",
                "vars" => array(
                    "image" => ( $challenge->getImage() ? $challengeService->generateImageUrl($challenge->getImage(), false) : $challengeService->generateDefaultImageUrl() ),
                    "thumbnail" => ( $challenge->getImage() ? $challengeService->generateImageUrl($challenge->getImage(), true) : $challengeService->generateDefaultImageUrl() ),
                    "title" => $title,
                    "description" => UTIL_String::truncate(strip_tags($challenge->getDescription()), 200, '...'),
                    "url" => array(
                        "routeName" => 'challenge.view',
                        "vars" => array('challengeId' => $challenge->getId())
                    ),
                    'iconClass' => 'ow_ic_challenge'
                )
            ),
            'view' => array(
                'iconClass' => 'ow_ic_calendar'
            ),
        );

        if ( $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY )
        {
            $data['params']['visibility'] = 4;
        }

        $challenge = new OW_Challenge('feed.action', array(
                'entityType' => 'challenge',
                'entityId' => $challengeId,
                'pluginKey' => 'challenge'
                ), $data);

        OW::getChallengeManager()->trigger($challenge);
    }

    public function addNewContentItem( BASE_CLASS_ChallengeCollector $challenge )
    {
        if ( !OW::getUser()->isAuthorized('challenge', 'add_challenge') )
        {
            return;
        }

        $resultArray = array(
            BASE_CMP_AddNewContent::DATA_KEY_ICON_CLASS => 'ow_ic_calendar',
            BASE_CMP_AddNewContent::DATA_KEY_URL => OW::getRouter()->urlForRoute('challenge.add'),
            BASE_CMP_AddNewContent::DATA_KEY_LABEL => OW::getLanguage()->text('challenge', 'add_new_link_label')
        );

        $challenge->add($resultArray);
    }

    public function adsEnabled( BASE_ChallengeCollector $challenge )
    {
        $challenge->add('challenge');
    }

    public function isPluginActive( OW_Challenge $challenge )
    {
        $challenge->setData(true);
    }

    public function addAuthLabels( BASE_CLASS_ChallengeCollector $challenge )
    {
        $language = OW::getLanguage();
        $challenge->add(
            array(
                'challenge' => array(
                    'label' => $language->text('challenge', 'auth_group_label'),
                    'actions' => array(
                        'add_challenge' => $language->text('challenge', 'auth_action_label_add_challenge'),
                        'view_challenge' => $language->text('challenge', 'auth_action_label_view_challenge'),
                        'add_comment' => $language->text('challenge', 'auth_action_label_add_comment')
                    )
                )
            )
        );
    }

    public function onUserDelete( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();

        if ( empty($params['deleteContent']) )
        {
            return;
        }

        $userId = $params['userId'];

        CHALLENGE_BOL_ChallengeService::getInstance()->deleteUserChallenges($userId);
    }

    public function privacyAddAction( BASE_CLASS_ChallengeCollector $challenge )
    {
        $language = OW::getLanguage();

        $action = array(
            'key' => 'challenge_view_attend_challenges',
            'pluginKey' => 'challenge',
            'label' => $language->text('challenge', 'privacy_action_view_attend_challenges'),
            'description' => '',
            'defaultValue' => 'everybody'
        );

        $challenge->add($action);
    }

    public function feedOnItemRenderActivity( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        $data = $challenge->getData();

        if ( $params['action']['entityType'] != 'challenge' )
        {
            return;
        }

        $challengeId = $params['action']['entityId'];
        $usersCount = CHALLENGE_BOL_ChallengeService::getInstance()->findChallengeUsersCount($challengeId, CHALLENGE_BOL_ChallengeService::USER_STATUS_YES);

        if ( $usersCount == 1 )
        {
            return;
        }

        $users = CHALLENGE_BOL_ChallengeService::getInstance()->findChallengeUsers($challengeId, CHALLENGE_BOL_ChallengeService::USER_STATUS_YES, null, 6);

        $userIds = array();

        foreach ( $users as $user )
        {
            $userIds[] = $user->getUserId();
        }

        $activityUserIds = array();

        foreach ( $params['activity'] as $activity )
        {
            if ( $activity['activityType'] == 'challenge-join' )
            {
                $activityUserIds[] = $activity['data']['userId'];
            }
        }

        $lastUserId = reset($activityUserIds);
        $follows = array_intersect($activityUserIds, $userIds);
        $notFollows = array_diff($userIds, $activityUserIds);
        $idlist = array_merge($follows, $notFollows);
        $idlist = array_slice($idlist, 0, 5);
        //$avatarList = new BASE_CMP_MiniAvatarUserList();
        //$avatarList->setEmptyListNoRender(true);

        /* if ( count($idlist) > 5 )
          {
          $avatarList->setViewMoreUrl(OW::getRouter()->urlForRoute('challenge.main_user_list', array('challengeId' => $challengeId)));
          } */

        $language = OW::getLanguage();

        //$avatarList = new BASE_CMP_MiniAvatarUserList($idlist);
        //$content = $avatarList->render();

        /* if ( $lastUserId )
        {
            $userName = BOL_UserService::getInstance()->getDisplayName($lastUserId);
            $userUrl = BOL_UserService::getInstance()->getUserUrl($lastUserId);
            $content = $language->text('challenge', 'feed_activity_joined', array('user' => '<a href="' . $userUrl . '">' . $userName . '</a>'));
        } */

        /* $data['assign']['activity'] = array('template' => 'activity',
          'vars' => array(
          'title' => $language->text('challenge', 'feed_activity_users', array('usersCount' => $usersCount)),
          'content' => $content
          )); */

        $userListData = array( 'userList' => array(
            'label' => array( 'key' => 'challenge+feed_activity_users', 'vars' => array('usersCount' => $usersCount) ),
            'ids' => $idlist
            ) );

        $data['content']['vars'] = array_merge($data['content']['vars'], $userListData);
        $challenge->setData($data);
    }

    public function feedOnCollectPrivacy( BASE_CLASS_ChallengeCollector $challenge )
    {
        $challenge->add(array('challenge-join', 'challenge_view_attend_challenges'));
    }

    public function feedOnCollectConfigurableActivity( BASE_CLASS_ChallengeCollector $challenge )
    {
        $language = OW::getLanguage();
        $challenge->add(array(
            'label' => $language->text('challenge', 'feed_content_label'),
            'activity' => '*:challenge'
        ));
    }

    public function onAddComment( OW_Challenge $e )
    {
        $params = $e->getParams();

        if ( empty($params['entityType']) || $params['entityType'] != 'challenge' )
        {
            return;
        }

        $entityId = $params['entityId'];
        $userId = $params['userId'];
        $commentId = $params['commentId'];
        $challenge = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($entityId);

        $challengeUrl = OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->id));

        $challengeImage = null;
        if ( !empty($challenge->image) )
        {
            $challengeImage = CHALLENGE_BOL_ChallengeService::getInstance()->generateImageUrl($challenge->image, true);
        }

        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($entityId);
        $ownerId = $challengeDto->userId;
        $userName = BOL_UserService::getInstance()->getDisplayName($ownerId);
        $userUrl = BOL_UserService::getInstance()->getUserUrl($ownerId);
        $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

        $string = OW::getLanguage()->text('challenge', 'feed_activity_comment_string', array('user' => $userEmbed));

        if ( !empty($challengeDto) && $userId == $challengeDto->userId )
        {
            $string = OW::getLanguage()->text('challenge', 'feed_activity_own_comment_string');
        }

        OW::getChallengeManager()->trigger(new OW_Challenge('feed.activity', array(
                'activityType' => 'comment',
                'activityId' => $commentId,
                'entityId' => $entityId,
                'entityType' => $params['entityType'],
                'userId' => $userId,
                'pluginKey' => 'challenge'
                ), array(
                'string' => $string,
                'line' => null
            )));


//    $string = OW::getLanguage()->text('challenge', 'email_notification_comment', array(
//            'userName' => BOL_UserService::getInstance()->getDisplayName($userId),
//            'userUrl' => BOL_UserService::getInstance()->getUserUrl($userId),
//            'url' => $challengeUrl,
//            'title' => strip_tags($challenge->title)
//        ));
//
//    $params = array(
//        'plugin' => 'challenge',
//        'action' => 'challenge-add_comment',
//        'string' => $string,
//        'avatar' => $challengeImage,
//        'content' => $comment->getMessage(),
//        'url' => $challengeUrl,
//        'time' => time(),
//        'userId' => $userId
//    );

        if ( $userId != $challenge->userId )
        {
            $avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($userId), true, true, false, false);
            $avatar = $avatars[$userId];

            $contentImage = array();

            if ( !empty($challengeImage) )
            {
                $contentImage = array('src' => $challengeImage);
            }

            $challenge = new OW_Challenge('notifications.add', array(
                    'pluginKey' => 'challenge',
                    'entityType' => $params['entityType'],
                    'entityId' => $params['entityId'],
                    'action' => 'challenge-add_comment',
                    'userId' => $challenge->userId,
                    'time' => time()
                    ), array(
                    'avatar' => $avatar,
                    'string' => array(
                        'key' => 'challenge+email_notification_comment',
                        'vars' => array(
                            'userName' => BOL_UserService::getInstance()->getDisplayName($userId),
                            'userUrl' => BOL_UserService::getInstance()->getUserUrl($userId),
                            'url' => $challengeUrl,
                            'title' => strip_tags($challenge->title)
                        )
                    ),
                    'url' => $challengeUrl,
                    'contentImage' => $contentImage
                ));

            OW::getChallengeManager()->trigger($challenge);
        }
    }

    public function feedOnLike( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();

        if ( $params['entityType'] != 'challenge' )
        {
            return;
        }

        $userId = (int) $params['userId'];
        $entityId = $params['entityId'];

        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($entityId);
        $ownerId = $challengeDto->userId;
        $userName = BOL_UserService::getInstance()->getDisplayName($ownerId);
        $userUrl = BOL_UserService::getInstance()->getUserUrl($ownerId);
        $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

        $string = OW::getLanguage()->text('challenge', 'feed_activity_challenge_string_like', array('user' => $userEmbed));

        if ( !empty($challengeDto) && $userId == $challengeDto->userId )
        {
            $string = OW::getLanguage()->text('challenge', 'feed_activity_challenge_string_like_own');
        }

        OW::getChallengeManager()->trigger(new OW_Challenge('feed.activity', array(
                'activityType' => 'like',
                'activityId' => $params['userId'],
                'entityId' => $params['entityId'],
                'entityType' => $params['entityType'],
                'userId' => $params['userId'],
                'pluginKey' => 'challenge'
                ), array(
                'string' => $string,
                'line' => null
            )));
    }

    public function quickLinks( BASE_CLASS_ChallengeCollector $challenge )
    {
        $service = CHALLENGE_BOL_ChallengeService::getInstance();
        $userId = OW::getUser()->getId();

        $challengesCount = $service->findUserParticipatedChallengesCount($userId);
        $invitesCount = $service->findUserInvitedChallengesCount($userId);

        if ( $challengesCount > 0 || $invitesCount > 0 )
        {
            $challenge->add(array(
                BASE_CMP_QuickLinksWidget::DATA_KEY_LABEL => OW::getLanguage()->text('challenge', 'common_list_type_joined_label'),
                BASE_CMP_QuickLinksWidget::DATA_KEY_URL => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'joined')),
                BASE_CMP_QuickLinksWidget::DATA_KEY_COUNT => $challengesCount,
                BASE_CMP_QuickLinksWidget::DATA_KEY_COUNT_URL => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'joined')),
                BASE_CMP_QuickLinksWidget::DATA_KEY_ACTIVE_COUNT => $invitesCount,
                BASE_CMP_QuickLinksWidget::DATA_KEY_ACTIVE_COUNT_URL => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'invited'))
            ));
        }
    }

    public function onAddChallenge( OW_Challenge $challenge )
    {
        OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeDao::CACHE_TAG_CHALLENGE_LIST));
    }

    public function onDeleteChallenge( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        $challengeId = !empty($params['challengeId']) ? $params['challengeId'] : null;

        OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeDao::CACHE_TAG_CHALLENGE_LIST));

        if ( isset($challengeId) )
        {
            OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeUserDao::CACHE_TAG_CHALLENGE_USER_LIST . $challengeId));
        }

        $challenge = new OW_Challenge('feed.delete_item', array('entityType' => 'challenge', 'entityId' => $challengeId));
        OW::getChallengeManager()->trigger($challenge);
    }

    public function onEditChallenge( OW_Challenge $challenge )
    {
        OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeDao::CACHE_TAG_CHALLENGE_LIST));
    }

    public function onChangeUserStatus( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        $challengeId = !empty($params['challengeId']) ? $params['challengeId'] : null;
        $userId = !empty($params['userId']) ? $params['userId'] : null;

        if ( !isset($challengeId) )
        {
            return;
        }

        OW::getCacheManager()->clean(array(CHALLENGE_BOL_ChallengeUserDao::CACHE_TAG_CHALLENGE_USER_LIST . $challengeId));

        if ( !isset($userId) )
        {
            return;
        }

        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($challengeId);

        $challengeUser = CHALLENGE_BOL_ChallengeService::getInstance()->findChallengeUser($challengeId, $userId);

        if ( empty($challengeDto) || empty($challengeUser) )
        {
            return;
        }

        if ( $challengeUser->getStatus() == CHALLENGE_BOL_ChallengeService::USER_STATUS_YES && $challengeDto->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_ANYBODY )
        {
            $userName = BOL_UserService::getInstance()->getDisplayName($challengeDto->getUserId());
            $userUrl = BOL_UserService::getInstance()->getUserUrl($challengeDto->getUserId());
            $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

            OW::getChallengeManager()->trigger(new OW_Challenge('feed.activity', array(
                    'activityType' => 'challenge-join',
                    'activityId' => $challengeUser->getId(),
                    'entityId' => $challengeDto->getId(),
                    'entityType' => 'challenge',
                    'userId' => $challengeUser->getUserId(),
                    'pluginKey' => 'challenge'
                    ), array(
                    'challengeId' => $challengeDto->getId(),
                    'userId' => $challengeUser->getUserId(),
                    'challengeUserId' => $challengeUser->getId(),
                    'string' => OW::getLanguage()->text('challenge', 'feed_actiovity_attend_string', array('user' => $userEmbed)),
                    'feature' => array()
                )));
        }
    }

    public function sosialSharingGetChallengeInfo( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        $data = $challenge->getData();
        $data['display'] = false;

        if ( empty($params['entityId']) )
        {
            return;
        }

        if ( $params['entityType'] == 'challenge' )
        {
            if ( !BOL_AuthorizationService::getInstance()->isActionAuthorizedForGuest('challenge', 'view_challenge') )
            {
                $challenge->setData($data);
                return;
            }

            $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($params['entityId']);
            
            if ( !empty($challengeDto) )
            {
                $data['display'] = $challengeDto->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_ANYBODY;
            }

            $challenge->setData($data);
        }
    }

    public function getContentMenu( OW_Challenge $challenge )
    {
        //$challenge->setData(CHALLENGE_BOL_ChallengeService::getInstance()->getContentMenu());
    }

    public function genericInit()
    {
        OW::getChallengeManager()->bind('notifications.collect_actions', array($this, 'onNotifyActions'));
        OW::getChallengeManager()->bind('challenge.invite_user', array($this, 'onUserInvite'));
        OW::getChallengeManager()->bind('feed.on_entity_add', array($this, 'feedEntityAdd'));
        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_AFTER_CHALLENGE_EDIT, array($this, 'afterChallengeEdit'));
        OW::getChallengeManager()->bind(BASE_CMP_AddNewContent::CHALLENGE_NAME, array($this, 'addNewContentItem'));
        OW::getChallengeManager()->bind('ads.enabled_plugins', array($this, 'adsEnabled'));
        OW::getChallengeManager()->bind('challenge.is_plugin_active', array($this, 'isPluginActive'));

        $credits = new CHALLENGE_CLASS_Credits();
        OW::getChallengeManager()->bind('usercredits.on_action_collect', array($credits, 'bindCreditActionsCollect'));

        OW::getChallengeManager()->bind('admin.add_auth_labels', array($this, 'addAuthLabels'));
        OW::getChallengeManager()->bind(OW_ChallengeManager::ON_USER_UNREGISTER, array($this, 'onUserDelete'));
        OW::getChallengeManager()->bind('plugin.privacy.get_action_list', array($this, 'privacyAddAction'));

        OW::getChallengeManager()->bind('feed.on_item_render', array($this, 'feedOnItemRenderActivity'));

        OW::getChallengeManager()->bind('feed.collect_privacy', array($this, 'feedOnCollectPrivacy'));
        OW::getChallengeManager()->bind('feed.collect_configurable_activity', array($this, 'feedOnCollectConfigurableActivity'));
        OW::getChallengeManager()->bind('base_add_comment', array($this, 'onAddComment'));
        OW::getChallengeManager()->bind('feed.after_like_added', array($this, 'feedOnLike'));
        OW::getChallengeManager()->bind(BASE_CMP_QuickLinksWidget::CHALLENGE_NAME, array($this, 'quickLinks'));
        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CREATE_CHALLENGE, array($this, 'onAddChallenge'));
        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_DELETE_CHALLENGE, array($this, 'onDeleteChallenge'));
        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_AFTER_CHALLENGE_EDIT, array($this, 'onEditChallenge'));

        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CHANGE_USER_STATUS, array($this, 'onChangeUserStatus'));

        OW::getChallengeManager()->bind('socialsharing.get_entity_info', array($this, 'sosialSharingGetChallengeInfo'));
    }

    public function init()
    {
        CHALLENGE_CLASS_InvitationHandler::getInstance()->init();
        //OW::getChallengeManager()->bind('challenge.get_content_menu', 'getContentMenu');
    }
}