<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Podyachev Evgeny <joker.OW2@gmail.com>
 * @package ow_plugins.challenge.classes
 * @since 1.0
 */

class CHALLENGE_CLASS_InvitationHandler
{
    const INVITATION_JOIN = 'challenge-join';

    /**
     * Singleton instance.
     *
     * @var CHALLENGE_CLASS_InvitationHandler
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return CHALLENGE_CLASS_InvitationHandler
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }



    private function __construct()
    {

    }

    public function onInvite( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();

        $challengeId = $params['challengeId'];

        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($params['challengeId']);
        $challengeUrl = OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challengeDto->id));

        $challengeTitle = UTIL_String::truncate($challengeDto->title, 100, '...');

        $userId = OW::getUser()->getId();
        $userDto = OW::getUser()->getUserObject();
        $userUrl = BOL_UserService::getInstance()->getUserUrlForUsername($userDto->username);
        $userDisplayName = BOL_UserService::getInstance()->getDisplayName($userId);
        $avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($userId));
        $avatar = $avatars[$userId];

        $users = array($userId);

        $stringAssigns = array(
            'challenge' => '<a href="' . $challengeUrl . '">' . $challengeTitle . '</a>'
        );

        $stringAssigns['user1'] = '<a href="' . $userUrl . '">' . $userDisplayName . '</a>';

        $contentImage = null;

        if ( !empty($challengeDto->image) )
        {
            $challengeSrc = CHALLENGE_BOL_ChallengeService::getInstance()->generateImageUrl($challengeDto->image, true);

            $contentImage = array(
                    'src' => $challengeSrc,
                    'url' => $challengeUrl,
                    'title' => $challengeTitle
                );
        }

        //$userCount = count($data['users']);
//        $userIds = array();
//        for ( $i = 0; $i < $userCount; $i++ )
//        {
//            $user = $data['users'][$i];
//            $stringAssigns['user' . ($i+1)] = '<a href="' . $user['url'] . '">' . $user['name'] . '</a>';
//
//            if ( $i >= 2 )
//            {
//                $userIds[] = $user['userId'];
//            }
//        }



        //$stringAssigns['otherUsers'] = '<a href="javascript://" onclick="OW.showUsers(' . json_encode($users) . ');">' .
        //OW::getLanguage()->text('challenge', 'invitation_join_string_other_users', array( 'count' => count($users) )) . '</a>';

        // $languageKey = $userCount > 2 ? 'invitation_join_string_many' : 'invitation_join_string_' . $userCount;
        $languageKey = 'invitation_join_string_' . 1;

        $invitationChallenge = new OW_Challenge('invitations.add', array(
            'pluginKey' => 'challenge',
            'entityType' => self::INVITATION_JOIN,
            'entityId' => $challengeDto->id,
            'userId' => $params['userId'],
            'time' => time(),
            'action' => 'challenge-invitation'
        ), array(
            'string' => array(
                'key' => 'challenge+'.$languageKey,
                'vars' => $stringAssigns
            ),
            'users' => $users,
            'avatar' => $avatar,
            'contentImage' => $contentImage
        ));


        OW::getChallengeManager()->trigger($invitationChallenge);
    }

    public function onItemRender( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();

        if ( $params['entityType'] != self::INVITATION_JOIN )
        {
            return;
        }

        $challengeId = (int) $params['entityId'];
        $data = $params['data'];

        $itemKey = $params['key'];
        
        $language = OW::getLanguage();

        $data['toolbar'] = array(
            array(
                'label' => $language->text('challenge', 'accept_request'),
                'id'=> 'toolbar_accept_' . $itemKey
            ),
            array(
                'label' => $language->text('challenge', 'ignore_request'),
                'id'=> 'toolbar_ignore_' . $itemKey
            )
        );

        $challenge->setData($data);

        $jsData = array(
            'challengeId' => $challengeId,
            'itemKey' => $itemKey
        );

        $js = UTIL_JsGenerator::newInstance();
        $js->jQueryChallenge("#toolbar_ignore_$itemKey", 'click',
                'OW.Invitation.send("challenges.ignore", e.data.challengeId).removeItem(e.data.itemKey);',
        array('e'), $jsData);

        $js->jQueryChallenge("#toolbar_accept_$itemKey", 'click',
                'OW.Invitation.send("challenges.accept", e.data.challengeId);
                 $("#toolbar_ignore_" + e.data.itemKey).hide();
                 $("#toolbar_accept_" + e.data.itemKey).hide();',
        array('e'), $jsData);

        OW::getDocument()->addOnloadScript($js->generateJs());
    }

    public function onChallengeDelete( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        $challengeId = $params['challengeId'];

        OW::getChallengeManager()->call('invitations.remove', array(
            'entityType' => 'challenge',
            'entityId' => $challengeId
        ));

        OW::getChallengeManager()->call('notifications.remove', array(
            'entityType' => 'challenge',
            'entityId' => $challengeId
        ));
    }

    public function onCommand( OW_Challenge $challenge )
    {
        if ( !OW::getUser()->isAuthenticated() )
        {
            return 'auth faild';
        }

        $params = $challenge->getParams();

        if ( !in_array($params['command'], array('challenges.accept', 'challenges.ignore')) )
        {
            return 'wrong command';
        }

        $challengeId = $params['data'];
        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($challengeId);

        $userId = OW::getUser()->getId();
        $jsResponse = UTIL_JsGenerator::newInstance();
        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();

        if ( empty($challengeDto) )
        {
            BOL_InvitationService::getInstance()->deleteInvitation(self::INVITATION_JOIN, $challengeId, $userId);
            return 'empty Challenge Id';
        }

        if ( $params['command'] == 'challenges.accept' )
        {
            $feedback = array('messageType' => 'error');
            $exit = false;
            $attendedStatus = 1;

            if ( $challengeService->canUserView($challengeId, $userId) )
            {
                $challengeDto = $challengeService->findChallenge($challengeId);

                if ( $challengeDto->getEndTimeStamp() < time() )
                {
                    $challengeService->deleteUserChallengeInvites((int)$challengeId, $userId);
                    $jsResponse->callFunction(array('OW', 'error'), array( OW::getLanguage()->text('challenge', 'user_status_updated') ));
                    $challenge->setData($jsResponse);
                    return;
                }

                $challengeUser = $challengeService->findChallengeUser($challengeId, $userId);

                if ( $challengeUser !== null && (int) $challengeUser->getStatus() === (int) $attendedStatus )
                {
                    $jsResponse->callFunction(array('OW', 'error'), array( OW::getLanguage()->text('challenge', 'user_status_not_changed_error') ));
                    $exit = true;
                }

                if ( $challengeDto->getUserId() == OW::getUser()->getId() && (int) $attendedStatus == CHALLENGE_BOL_ChallengeService::USER_STATUS_NO )
                {
                    $jsResponse->callFunction(array('OW', 'error'), array( OW::getLanguage()->text('challenge', 'user_status_author_cant_leave_error') ));
                    $exit = true;
                }

                if ( !$exit )
                {
                    $challengeUserDto = CHALLENGE_BOL_ChallengeService::getInstance()->addChallengeUser($userId, $challengeId, $attendedStatus );

                    if( !empty( $challengeUserDto ) )
                    {
                        $e = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CHANGE_USER_STATUS, array('challengeId' => $challengeDto->id, 'userId' => $challengeUserDto->userId));
                        OW::getChallengeManager()->trigger($e);

                        $jsResponse->callFunction(array('OW', 'info'), array( OW::getLanguage()->text('challenge', 'user_status_updated') ));
                        BOL_InvitationService::getInstance()->deleteInvitation(self::INVITATION_JOIN, $challengeId, $userId);
                    }
                    else
                    {
                        $jsResponse->callFunction(array('OW', 'error'), array( OW::getLanguage()->text('challenge', 'user_status_update_error') ));
                    }
                }
            }
            else
            {
                $jsResponse->callFunction(array('OW', 'error'), array( OW::getLanguage()->text('challenge', 'user_status_update_error') ));
            }
        }
        else if ( $params['command'] == 'challenges.ignore' )
        {
            $challengeService->deleteUserChallengeInvites((int)$challengeId, $userId);
            $jsResponse->callFunction(array('OW', 'info'), array( OW::getLanguage()->text('challenge', 'user_status_updated') ));
            BOL_InvitationService::getInstance()->deleteInvitation(self::INVITATION_JOIN, $challengeId, $userId);
        }

        $challenge->setData($jsResponse);
    }

    public function init()
    {
        OW::getChallengeManager()->bind('challenge.invite_user', array($this, 'onInvite'));
        OW::getChallengeManager()->bind('invitations.on_item_render', array($this, 'onItemRender'));
        OW::getChallengeManager()->bind(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_DELETE_CHALLENGE, array($this, 'onChallengeDelete'));

        OW::getChallengeManager()->bind('invitations.on_command', array($this, 'onCommand'));
    }
}