<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * User console component class.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>, Podyachev Evgeny <joker.OW2@gmail.com>
 * @package ow.ow_system_plugins.base.components
 * @since 1.0
 */
class CHALLENGE_CMP_ChallengeUsers extends OW_Component
{
    private $challengeService;

    /**
     * @return Constructor.
     */
    public function __construct( $challengeId )
    {
        parent::__construct();

        $this->challengeService = CHALLENGE_BOL_ChallengeService::getInstance();

        $challenge = $this->challengeService->findChallenge($challengeId);

        if ( $challenge === null )
        {
            $this->setVisible(false);
        }

        // challenge users info
        $this->addUserList($challenge, CHALLENGE_BOL_ChallengeService::USER_STATUS_YES);
        $this->addUserList($challenge, CHALLENGE_BOL_ChallengeService::USER_STATUS_MAYBE);
        $this->addUserList($challenge, CHALLENGE_BOL_ChallengeService::USER_STATUS_NO);
        $this->assign('userLists', $this->userLists);
        $this->addComponent('userListMenu', new BASE_CMP_WidgetMenu($this->userListMenu));
    }
    private $userLists;
    private $userListMenu;

    private function addUserList( CHALLENGE_BOL_Challenge $challenge, $status )
    {
        $configs = $this->challengeService->getConfigs();

        $language = OW::getLanguage();
        $listTypes = $this->challengeService->getUserListsArray();
        $serviceConfigs = $this->challengeService->getConfigs();
        $userList = $this->challengeService->findChallengeUsers($challenge->getId(), $status, null, $configs[CHALLENGE_BOL_ChallengeService::CONF_CHALLENGE_USERS_COUNT]);
        $usersCount = $this->challengeService->findChallengeUsersCount($challenge->getId(), $status);

        $idList = array();

        /* @var $challengeUser CHALLENGE_BOL_ChallengeUser */
        foreach ( $userList as $challengeUser )
        {
            $idList[] = $challengeUser->getUserId();
        }

        $usersCmp = new BASE_CMP_AvatarUserList($idList);

        $linkId = UTIL_HtmlTag::generateAutoId('link');
        $contId = UTIL_HtmlTag::generateAutoId('cont');

        $this->userLists[] = array(
            'contId' => $contId,
            'cmp' => $usersCmp->render(),
            'bottomLinkEnable' => ($usersCount > $serviceConfigs[CHALLENGE_BOL_ChallengeService::CONF_CHALLENGE_USERS_COUNT]),
            'toolbarArray' => array(
                array(
                    'label' => $language->text('challenge', 'avatar_user_list_bottom_link_label', array('count' => $usersCount)),
                    'href' => OW::getRouter()->urlForRoute('challenge.user_list', array('challengeId' => $challenge->getId(), 'list' => $listTypes[(int) $status]))
                )
            )
        );

        $this->userListMenu[] = array(
            'label' => $language->text('challenge', 'avatar_user_list_link_label_' . $status),
            'id' => $linkId,
            'contId' => $contId,
            'active' => ( sizeof($this->userListMenu) < 1 ? true : false )
        );
    }
}