<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * User console component class.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>,
 * @package ow.ow_plugins.challenge.components
 * @since 1.0
 */
class CHALLENGE_CMP_MyChallenges extends BASE_CLASS_Widget
{

    /**
     * @return Constructor.
     */
    public function __construct( BASE_CLASS_WidgetParameter $paramsObj )
    {
        parent::__construct();

        $params = $paramsObj->customParamList;

        if ( !OW::getUser()->isAuthenticated() )
        {
            $this->setVisible(false);
            return;
        }

        $language = OW::getLanguage();
        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();

        $userChallenges = $challengeService->findUserChallenges(OW::getUser()->getId(), null, $params['challenges_count']);
        $partChallenges = $challengeService->findUserParticipatedChallenges(OW::getUser()->getId(), null, $params['challenges_count']);

        if ( empty($userChallenges) && empty($partChallenges) || !OW::getUser()->isAuthorized('challenge', 'view_challenge') )
        {
            $this->setVisible(false);
            return;
        }

        if ( !OW::getUser()->isAuthorized('challenge', 'add_challenge') )
        {
            $this->assign('noMenu', true);
        }

        if ( !$paramsObj->customizeMode )
        {
            $menuArray = array(
                array(
                    'label' => $language->text('challenge', 'dashboard_widget_menu_part_challenges_label'),
                    'id' => 'challenge_menu_part_id',
                    'contId' => 'challenge_menu_part_cont',
                    'active' => true
                ),
                array(
                    'label' => $language->text('challenge', 'dashboard_widget_menu_my_challenges_label'),
                    'id' => 'challenge_menu_my_id',
                    'contId' => 'challenge_menu_my_cont',
                    'active' => false
                )
            );

            $this->addComponent('listMenu', new BASE_CMP_WidgetMenu($menuArray));
        }
        else
        {
            $this->assign('noMenu', true);
        }

        $this->assign('my_challenges', $challengeService->getListingDataWithToolbar($userChallenges));

        $toolbarArray = array();

        if ( $challengeService->findUsersChallengesCount(OW::getUser()->getId()) > $params['challenges_count'] )
        {
            $toolbarArray['my'] = array(array('href' => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'created')), 'label' => $language->text('challenge', 'view_all_label')));
        }

        $this->assign('part_challenges', $challengeService->getListingDataWithToolbar($partChallenges));

        if ( $challengeService->findUserParticipatedChallengesCount(OW::getUser()->getId()) > $params['challenges_count'] )
        {
            $toolbarArray['part'] = array(array('href' => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'joined')), 'label' => $language->text('challenge', 'view_all_label')));
        }

        $this->assign('toolbars', $toolbarArray);
    }

    public static function getSettingList()
    {
        $challengeConfigs = CHALLENGE_BOL_ChallengeService::getInstance()->getConfigs();
        $settingList = array();
        $settingList['challenges_count'] = array(
            'presentation' => self::PRESENTATION_SELECT,
            'label' => OW::getLanguage()->text('challenge', 'cmp_widget_challenges_count'),
            'optionList' => $challengeConfigs[CHALLENGE_BOL_ChallengeService::CONF_WIDGET_CHALLENGES_COUNT_OPTION_LIST],
            'value' => $challengeConfigs[CHALLENGE_BOL_ChallengeService::CONF_WIDGET_CHALLENGES_COUNT]
        );

        return $settingList;
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_SHOW_TITLE => true,
            self::SETTING_TITLE => OW::getLanguage()->text('challenge', 'my_challenges_widget_block_cap_label'),
            self::SETTING_WRAP_IN_BOX => true,
            self::SETTING_ICON => self::ICON_CALENDAR
        );
    }

    public static function getAccess()
    {
        return self::ACCESS_MEMBER;
    }
}