<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * User console component class.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow.ow_plugins.challenge.components
 * @since 1.0
 */
class CHALLENGE_CMP_UpcomingChallenges extends BASE_CLASS_Widget
{

    /**
     * @return Constructor.
     */
    public function __construct( BASE_CLASS_WidgetParameter $paramsObj )
    {
        parent::__construct();

        $params = $paramsObj->customParamList;

        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();
        $challenges = $challengeService->findPublicChallenges(null, $params['challenges_count']);
        $count = $challengeService->findPublicChallengesCount();

        if ( ( !OW::getUser()->isAuthenticated() || !OW::getUser()->isAuthorized('challenge', 'add_challenge') ) && $count == 0 )
        {
            $this->setVisible(false);
            return;
        }

        $this->assign('challenges', $challengeService->getListingDataWithToolbar($challenges));
        $this->assign('no_content_message', OW::getLanguage()->text('challenge', 'no_index_challenges_label', array('url' => OW::getRouter()->urlForRoute('challenge.add'))));

        if ( $challengeService->findPublicChallengesCount() > $params['challenges_count'] )
        {
            $toolbarArray = array(array('href' => OW::getRouter()->urlForRoute('challenge.view_challenge_list', array('list' => 'latest')), 'label' => OW::getLanguage()->text('challenge', 'view_all_label')));
            $this->assign('toolbar', $toolbarArray);
        }
    }

    public static function getSettingList()
    {
        $challengeConfigs = CHALLENGE_BOL_ChallengeService::getInstance()->getConfigs();
        $settingList = array();
        $settingList['challenges_count'] = array(
            'presentation' => self::PRESENTATION_SELECT,
            'label' => OW::getLanguage()->text('challenge', 'cmp_widget_challenges_count'),
            'optionList' => $challengeConfigs[CHALLENGE_BOL_ChallengeService::CONF_WIDGET_CHALLENGES_COUNT_OPTION_LIST],
            'value' => $challengeConfigs[CHALLENGE_BOL_ChallengeService::CONF_WIDGET_CHALLENGES_COUNT]
        );

        return $settingList;
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_SHOW_TITLE => true,
            self::SETTING_TITLE => OW::getLanguage()->text('challenge', 'up_challenges_widget_block_cap_label'),
            self::SETTING_WRAP_IN_BOX => false,
            self::SETTING_ICON => self::ICON_CALENDAR
        );
    }

    public static function getAccess()
    {
        return self::ACCESS_ALL;
    }
}