<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO CHALLENGE SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Sardar Madumarov <madumarov@gmail.com>, Podyachev Evgeny <joker.OW2@gmail.com>
 * @package ow_plugins.challenge.controllers
 * @since 1.0
 */
class CHALLENGE_CTRL_Base extends OW_ActionController
{
    /**
     * @var CHALLENGE_BOL_ChallengeService
     */
    private $challengeService;

    public function __construct()
    {
        parent::__construct();
        $this->challengeService = CHALLENGE_BOL_ChallengeService::getInstance();
    }

    /**
     * Add new challenge controller
     */
    public function add()
    {
        $language = OW::getLanguage();
        $this->setPageTitle($language->text('challenge', 'add_page_title'));
        $this->setPageHeading($language->text('challenge', 'add_page_heading'));
        $this->setPageHeadingIconClass('ow_ic_add');

        OW::getDocument()->setDescription(OW::getLanguage()->text('challenge', 'add_challenge_meta_description'));

        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'challenge', 'main_menu_item');

        // check permissions for this page
        if ( !OW::getUser()->isAuthenticated() || !OW::getUser()->isAuthorized('challenge', 'add_challenge') )
        {
            $this->assign('err_msg', OW::getLanguage()->text('base', 'authorization_failed_feedback'));
            return;
        }

        $challengeParams = array('pluginKey' => 'challenge', 'action' => 'add_challenge');
        $credits = OW::getChallengeManager()->call('usercredits.check_balance', $challengeParams);

        if ( $credits === false )
        {
            $this->assign('err_msg', OW::getChallengeManager()->call('usercredits.error_message', $challengeParams));
            return;
        }

        $form = new ChallengeAddForm('challenge_add');

        if ( date('n', time()) == 12 && date('j', time()) == 31 )
        {
            $defaultDate = (date('Y', time()) + 1) . '/1/1';
        }
        else if ( ( date('j', time()) + 1 ) > date('t') )
        {
            $defaultDate = date('Y', time()) . '/' . ( date('n', time()) + 1 ) . '/1';
        }
        else
        {
            $defaultDate = date('Y', time()) . '/' . date('n', time()) . '/' . ( date('j', time()) + 1 );
        }

        $form->getElement('start_date')->setValue($defaultDate);
        $form->getElement('end_date')->setValue($defaultDate);
        $form->getElement('start_time')->setValue('all_day');
        $form->getElement('end_time')->setValue('all_day');

        $checkboxId = UTIL_HtmlTag::generateAutoId('chk');
        $tdId = UTIL_HtmlTag::generateAutoId('td');
        $this->assign('tdId', $tdId);
        $this->assign('chId', $checkboxId);

        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin("challenge")->getStaticJsUrl() . 'challenge.js');
        OW::getDocument()->addOnloadScript("new challengeAddForm(". json_encode(array('checkbox_id' => $checkboxId, 'end_date_id' => $form->getElement('end_date')->getId(), 'tdId' => $tdId )) .")");

        if ( OW::getRequest()->isPost() )
        {
            if ( !empty($_POST['endDateFlag']) )
            {
                $this->assign('endDateFlag', true);
            }

            if ( $form->isValid($_POST) )
            {
                $data = $form->getValues();
                
                $serviceChallenge = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_BEFORE_CHALLENGE_CREATE, array(), $data);
                OW::getChallengeManager()->trigger($serviceChallenge);
                $data = $serviceChallenge->getData();
                
                $dateArray = explode('/', $data['start_date']);

                $startStamp = mktime(0, 0, 0, $dateArray[1], $dateArray[2], $dateArray[0]);

                if ( $data['start_time'] != 'all_day' )
                {
                    $startStamp = mktime($data['start_time']['hour'], $data['start_time']['minute'], 0, $dateArray[1], $dateArray[2], $dateArray[0]);
                }

                if ( !empty($_POST['endDateFlag']) && !empty($data['end_date']) )
                {
                    $dateArray = explode('/', $data['end_date']);
                    $endStamp = mktime(0, 0, 0, $dateArray[1], $dateArray[2], $dateArray[0]);

                    $endStamp = strtotime("+1 day", $endStamp);

                    if ( $data['end_time'] != 'all_day' )
                    {
                        $hour = 0;
                        $min = 0;

                        if( $data['end_time'] != 'all_day' )
                        {
                            $hour = $data['end_time']['hour'];
                            $min = $data['end_time']['minute'];
                        }

                        $dateArray = explode('/', $data['end_date']);
                        $endStamp = mktime($hour, $min, 0, $dateArray[1], $dateArray[2], $dateArray[0]);
                    }
                }
                
                $imageValid = true;
                $datesAreValid = true;
                $imagePosted = false;

                if ( !empty($_FILES['image']['name']) )
                {
                    if ( (int) $_FILES['image']['error'] !== 0 || !is_uploaded_file($_FILES['image']['tmp_name']) || !UTIL_File::validateImage($_FILES['image']['name']) )
                    {
                        $imageValid = false;
                        OW::getFeedback()->error($language->text('base', 'not_valid_image'));
                    }
                    else
                    {
                        $imagePosted = true;
                    }
                }

                if ( empty($endStamp) )
                {
                    $endStamp = strtotime("+1 day", $startStamp);
                    $endStamp = mktime(0, 0, 0, date('n',$endStamp), date('j',$endStamp), date('Y',$endStamp));
                }

                if ( !empty($endStamp) && $endStamp < $startStamp )
                {
                    $datesAreValid = false;
                    OW::getFeedback()->error($language->text('challenge', 'add_form_invalid_end_date_error_message'));
                }

                if ( $imageValid && $datesAreValid )
                {
                    $challenge = new CHALLENGE_BOL_Challenge();
                    $challenge->setStartTimeStamp($startStamp);
                    $challenge->setEndTimeStamp($endStamp);
                    $challenge->setCreateTimeStamp(time());
                    $challenge->setTitle(htmlspecialchars($data['title']));
                    $challenge->setLocation(UTIL_HtmlTag::autoLink(strip_tags($data['location'])));
                    $challenge->setWhoCanView((int) $data['who_can_view']);
                    $challenge->setWhoCanInvite((int) $data['who_can_invite']);
					$challenge->setWhoCanInvite((int) $data['status']);
                    $challenge->setDescription($data['desc']);
                    $challenge->setUserId(OW::getUser()->getId());
                    $challenge->setEndDateFlag( !empty($_POST['endDateFlag']) );
                    $challenge->setStartTimeDisable( $data['start_time'] == 'all_day' );
                    $challenge->setEndTimeDisable( $data['end_time'] == 'all_day' );

                    if ( $imagePosted )
                    {
                        $challenge->setImage(uniqid());
                    }
                    
                    $serviceChallenge = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CREATE_CHALLENGE, array('challengeDto' => $challenge));
                    OW::getChallengeManager()->trigger($serviceChallenge);

                    $this->challengeService->saveChallenge($challenge);
                    
                    if ( $imagePosted )
                    {
                        $this->challengeService->saveChallengeImage($_FILES['image']['tmp_name'], $challenge->getImage());
                    }

                    $challengeUser = new CHALLENGE_BOL_ChallengeUser();
                    $challengeUser->setChallengeId($challenge->getId());
                    $challengeUser->setUserId(OW::getUser()->getId());
                    $challengeUser->setTimeStamp(time());
                    $challengeUser->setStatus(CHALLENGE_BOL_ChallengeService::USER_STATUS_YES);
                    $this->challengeService->saveChallengeUser($challengeUser);
                    
                    OW::getFeedback()->info($language->text('challenge', 'add_form_success_message'));

                    if ( $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_ANYBODY )
                    {
                        $challengeObj = new OW_Challenge('feed.action', array(
                                'pluginKey' => 'challenge',
                                'entityType' => 'challenge',
                                'entityId' => $challenge->getId(),
                                'userId' => $challenge->getUserId()
                            ));
                        OW::getChallengeManager()->trigger($challengeObj);
                    }

                    if ( $credits === true )
                    {
                        OW::getChallengeManager()->call('usercredits.track_action', $challengeParams);
                    }
                    
                    $serviceChallenge = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_AFTER_CREATE_CHALLENGE, array('challengeDto' => $challenge));
                    OW::getChallengeManager()->trigger($serviceChallenge);
                    
                    $this->redirect(OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->getId())));
                }
            }
        }

        if( empty($_POST['endDateFlag']) )
        {
            //$form->getElement('start_time')->addAttribute('disabled', 'disabled');
            //$form->getElement('start_time')->addAttribute('style', 'display:none;');

            $form->getElement('end_date')->addAttribute('disabled', 'disabled');
            $form->getElement('end_date')->addAttribute('style', 'display:none;');

            $form->getElement('end_time')->addAttribute('disabled', 'disabled');
            $form->getElement('end_time')->addAttribute('style', 'display:none;');
        }

        $this->addForm($form);
    }

    /**
     * Get challenge by params(challengeId)
     * 
     * @param array $params
     * @return CHALLENGE_BOL_Challenge 
     */
    private function getChallengeForParams( $params )
    {
        if ( empty($params['challengeId']) )
        {
            throw new Redirect404Exception();
        }

        $challenge = $this->challengeService->findChallenge($params['challengeId']);

        if ( $challenge === null )
        {
            throw new Redirect404Exception();
        }

        return $challenge;
    }

    /**
     * Update challenge controller
     * 
     * @param array $params 
     */
    public function estado($params)
    {
        print($params);
    }
	
	public function edit( $params )
    {
        $challenge = $this->getChallengeForParams($params);
        $language = OW::getLanguage();
        $form = new ChallengeAddForm('challenge_edit');

        $form->getElement('title')->setValue($challenge->getTitle());
        $form->getElement('desc')->setValue($challenge->getDescription());
        $form->getElement('location')->setValue($challenge->getLocation());
        $form->getElement('who_can_view')->setValue($challenge->getWhoCanView());
        $form->getElement('who_can_invite')->setValue($challenge->getWhoCanInvite());
        $form->getElement('status')->setValue($challenge->getStatus());

        $startTimeArray = array('hour' => date('G', $challenge->getStartTimeStamp()), 'minute' => date('i', $challenge->getStartTimeStamp()));
        $form->getElement('start_time')->setValue($startTimeArray);

        $startDate = date('Y', $challenge->getStartTimeStamp()) . '/' . date('n', $challenge->getStartTimeStamp()) . '/' . date('j', $challenge->getStartTimeStamp());
        $form->getElement('start_date')->setValue($startDate);

        if ( $challenge->getEndTimeStamp() !== null )
        {
            $endTimeArray = array('hour' => date('G', $challenge->getEndTimeStamp()), 'minute' => date('i', $challenge->getEndTimeStamp()));
            $form->getElement('end_time')->setValue($endTimeArray);


            $endTimeStamp = $challenge->getEndTimeStamp();
            if ( $challenge->getEndTimeDisable() )
            {
                $endTimeStamp = strtotime("-1 day", $endTimeStamp);
            }

            $endDate = date('Y', $endTimeStamp) . '/' . date('n', $endTimeStamp) . '/' . date('j', $endTimeStamp);
            $form->getElement('end_date')->setValue($endDate);
        }

        if ( $challenge->getStartTimeDisable() )
        {
            $form->getElement('start_time')->setValue('all_day');
        }

        if ( $challenge->getEndTimeDisable() )
        {
            $form->getElement('end_time')->setValue('all_day');
        }

        $form->getSubmitElement('submit')->setValue(OW::getLanguage()->text('challenge', 'edit_form_submit_label'));

        $checkboxId = UTIL_HtmlTag::generateAutoId('chk');
        $tdId = UTIL_HtmlTag::generateAutoId('td');
        $this->assign('tdId', $tdId);
        $this->assign('chId', $checkboxId);
        
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin("challenge")->getStaticJsUrl() . 'challenge.js');
        OW::getDocument()->addOnloadScript("new challengeAddForm(". json_encode(array('checkbox_id' => $checkboxId, 'end_date_id' => $form->getElement('end_date')->getId(), 'tdId' => $tdId )) .")");

        if ( $challenge->getImage() )
        {
            $this->assign('imgsrc', $this->challengeService->generateImageUrl($challenge->getImage(), true));
        }

        $endDateFlag = $challenge->getEndDateFlag();

        if ( OW::getRequest()->isPost() )
        {
            $endDateFlag = !empty($_POST['endDateFlag']);

            //$this->assign('endDateFlag', !empty($_POST['endDateFlag']));

            if ( $form->isValid($_POST) )
            {
                $data = $form->getValues();
                
                $serviceChallenge = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_BEFORE_CHALLENGE_EDIT, array('challengeId' => $challenge->id), $data);
                OW::getChallengeManager()->trigger($serviceChallenge);
                $data = $serviceChallenge->getData();
                
                $dateArray = explode('/', $data['start_date']);

                $startStamp = mktime(0, 0, 0, $dateArray[1], $dateArray[2], $dateArray[0]);

                if ( $data['start_time'] != 'all_day' )
                {
                    $startStamp = mktime($data['start_time']['hour'], $data['start_time']['minute'], 0, $dateArray[1], $dateArray[2], $dateArray[0]);
                }

                if ( !empty($_POST['endDateFlag']) && !empty($data['end_date']) )
                {
                        $dateArray = explode('/', $data['end_date']);
                        $endStamp = mktime(0, 0, 0, $dateArray[1], $dateArray[2], $dateArray[0]);
                        $endStamp = strtotime("+1 day", $endStamp);

                        if ( $data['end_time'] != 'all_day' )
                        {
                            $hour = 0;
                            $min = 0;

                            if( $data['end_time'] != 'all_day' )
                            {
                                $hour = $data['end_time']['hour'];
                                $min = $data['end_time']['minute'];
                            }

                            $dateArray = explode('/', $data['end_date']);
                            $endStamp = mktime($hour, $min, 0, $dateArray[1], $dateArray[2], $dateArray[0]);
                        }
                }

                $challenge->setStartTimeStamp($startStamp);
                
                if ( empty($endStamp) )
                {
                    $endStamp = strtotime("+1 day", $startStamp);
                    $endStamp = mktime(0, 0, 0, date('n',$endStamp), date('j',$endStamp), date('Y',$endStamp));
                }
                
                if ( $startStamp > $endStamp )
                {
                    OW::getFeedback()->error($language->text('challenge', 'add_form_invalid_end_date_error_message'));
                    $this->redirect();
                }
                else
                {
                    $challenge->setEndTimeStamp($endStamp);

                    if ( !empty($_FILES['image']['name']) )
                    {
                        if ( (int) $_FILES['image']['error'] !== 0 || !is_uploaded_file($_FILES['image']['tmp_name']) || !UTIL_File::validateImage($_FILES['image']['name']) )
                        {
                            OW::getFeedback()->error($language->text('base', 'not_valid_image'));
                            $this->redirect();
                        }
                        else
                        {
                            $challenge->setImage(uniqid());
                            $this->challengeService->saveChallengeImage($_FILES['image']['tmp_name'], $challenge->getImage());

                        }
                    }
                                        
                    $challenge->setTitle(htmlspecialchars($data['title']));
                    $challenge->setLocation(UTIL_HtmlTag::autoLink(strip_tags($data['location'])));
                    $challenge->setWhoCanView((int) $data['who_can_view']);
                    $challenge->setWhoCanInvite((int) $data['who_can_invite']);
					$challenge->setStatus((int) $data['status']);
                    $challenge->setDescription($data['desc']);
                    $challenge->setEndDateFlag(!empty($_POST['endDateFlag']));
                    $challenge->setStartTimeDisable( $data['start_time'] == 'all_day' );
                    $challenge->setEndTimeDisable( $data['end_time'] == 'all_day' );

                    $this->challengeService->saveChallenge($challenge);
                    
                    $e = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_AFTER_CHALLENGE_EDIT, array('challengeId' => $challenge->id));
                    OW::getChallengeManager()->trigger($e);
                    
                    OW::getFeedback()->info($language->text('challenge', 'edit_form_success_message'));
                    $this->redirect(OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->getId())));
                }
            }
        }

        if( !$endDateFlag )
        {
           // $form->getElement('start_time')->addAttribute('disabled', 'disabled');
           // $form->getElement('start_time')->addAttribute('style', 'display:none;');

            $form->getElement('end_date')->addAttribute('disabled', 'disabled');
            $form->getElement('end_date')->addAttribute('style', 'display:none;');

            $form->getElement('end_time')->addAttribute('disabled', 'disabled');
            $form->getElement('end_time')->addAttribute('style', 'display:none;');
        }

        $this->assign('endDateFlag', $endDateFlag);

        $this->setPageHeading($language->text('challenge', 'edit_page_heading'));
        $this->setPageTitle($language->text('challenge', 'edit_page_title'));
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'challenge', 'main_menu_item');
        $this->addForm($form);
    }

    /**
     * Delete challenge controller
     * 
     * @param array $params 
     */
    public function delete( $params )
    {
        $challenge = $this->getChallengeForParams($params);

        if ( !OW::getUser()->isAuthenticated() || ( OW::getUser()->getId() != $challenge->getUserId() && !OW::getUser()->isAuthorized('challenge') ) )
        {
            throw new Redirect403Exception();
        }

        $this->challengeService->deleteChallenge($challenge->getId());
        OW::getFeedback()->info(OW::getLanguage()->text('challenge', 'delete_success_message'));
        $this->redirect(OW::getRouter()->urlForRoute('challenge.main_menu_route'));
    }

    
    /**
     * View challenge controller
     * 
     * @param array $params
     */
    public function view( $params )
    {
        $challenge = $this->getChallengeForParams($params);

        $cmpId = UTIL_HtmlTag::generateAutoId('cmp');

        $this->assign('contId', $cmpId);

        if ( !OW::getUser()->isAuthorized('challenge', 'view_challenge') && $challenge->getUserId() != OW::getUser()->getId() )
        {
            $this->assign('authErrorText', OW::getLanguage()->text('challenge', 'challenge_view_permission_error_message'));
            return;
        }

        // guest gan't view private challenges
        if ( (int) $challenge->getWhoCanView() === CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY && !OW::getUser()->isAuthenticated() )
        {
            $this->redirect(OW::getRouter()->urlForRoute('challenge.private_challenge', array('challengeId' => $challenge->getId())));
        }

        $challengeInvite = $this->challengeService->findChallengeInvite($challenge->getId(), OW::getUser()->getId());
        $challengeUser = $this->challengeService->findChallengeUser($challenge->getId(), OW::getUser()->getId());

        // check if user can view challenge
        if ( (int) $challenge->getWhoCanView() === CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY && $challengeUser === null && $challengeInvite === null && !OW::getUser()->isAuthorized('challenge') )
        {
            $this->redirect(OW::getRouter()->urlForRoute('challenge.private_challenge', array('challengeId' => $challenge->getId())));
        }

        if ( OW::getUser()->isAuthorized('challenge') || OW::getUser()->getId() == $challenge->getUserId() )
        {
            $this->assign('editArray', array(
                'edit' => array('url' => OW::getRouter()->urlForRoute('challenge.edit', array('challengeId' => $challenge->getId())), 'label' => OW::getLanguage()->text('challenge', 'edit_button_label')),
                'delete' =>
                array(
                    'url' => OW::getRouter()->urlForRoute('challenge.delete', array('challengeId' => $challenge->getId())),
                    'label' => OW::getLanguage()->text('challenge', 'delete_button_label'),
                    'confirmMessage' => OW::getLanguage()->text('challenge', 'delete_confirm_message')
                ),
                )
            );
        }
        
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'challenge', 'main_menu_item');
        
        $this->setPageHeading($challenge->getTitle());
        $this->setPageTitle(OW::getLanguage()->text('challenge', 'challenge_view_page_heading', array('challenge_title' => $challenge->getTitle())));
        $this->setPageHeadingIconClass('ow_ic_calendar');
        OW::getDocument()->setDescription(UTIL_String::truncate(strip_tags($challenge->getDescription()), 200, '...'));

        $infoArray = array(
            'id' => $challenge->getId(),
            'image' => ( $challenge->getImage() ? $this->challengeService->generateImageUrl($challenge->getImage(), false) : null ),
            'date' => UTIL_DateTime::formatSimpleDate($challenge->getStartTimeStamp(), $challenge->getStartTimeDisable()),
            'endDate' => $challenge->getEndTimeStamp() === null || !$challenge->getEndDateFlag() ? null : UTIL_DateTime::formatSimpleDate($challenge->getEndTimeDisable() ? strtotime("-1 day", $challenge->getEndTimeStamp()) : $challenge->getEndTimeStamp(),$challenge->getEndTimeDisable()),
            'location' => $challenge->getLocation(),
            'desc' => UTIL_HtmlTag::autoLink($challenge->getDescription()),
            'title' => $challenge->getTitle(),
            'creatorName' => BOL_UserService::getInstance()->getDisplayName($challenge->getUserId()),
            'creatorLink' => BOL_UserService::getInstance()->getUserUrl($challenge->getUserId()),
			'status'=>$challenge->getStatus()
        );

        $this->assign('info', $infoArray);

        // challenge attend form
        if ( OW::getUser()->isAuthenticated() && $challenge->getEndTimeStamp() > time() )
        {
            if ( $challengeUser !== null )
            {
                $this->assign('currentStatus', OW::getLanguage()->text('challenge', 'user_status_label_' . $challengeUser->getStatus()));
            }
            $this->addForm(new AttendForm($challenge->getId(), $cmpId));

            $onloadJs = "
                var \$context = $('#" . $cmpId . "');
                $('#challenge_attend_yes_btn').click(
                    function(){
                        $('input[name=attend_status]', \$context).val(" . CHALLENGE_BOL_ChallengeService::USER_STATUS_YES . ");
                    }
                );
                $('#challenge_attend_maybe_btn').click(
                    function(){
                        $('input[name=attend_status]', \$context).val(" . CHALLENGE_BOL_ChallengeService::USER_STATUS_MAYBE . ");
                    }
                );
                $('#challenge_attend_no_btn').click(
                    function(){
                        $('input[name=attend_status]', \$context).val(" . CHALLENGE_BOL_ChallengeService::USER_STATUS_NO . ");
                    }
                );

                $('.current_status a', \$context).click(
                    function(){
                        $('.attend_buttons .buttons', \$context).fadeIn(500);
                    }
                );
            ";

            OW::getDocument()->addOnloadScript($onloadJs);
        }
        else
        {
            $this->assign('no_attend_form', true);
        }
        
        if ( $challenge->getEndTimeStamp() > time() && ((int) $challenge->getUserId() === OW::getUser()->getId() || ( (int) $challenge->getWhoCanInvite() === CHALLENGE_BOL_ChallengeService::CAN_INVITE_PARTICIPANT && $challengeUser !== null) ) )
        {
            $params = array(
                $challenge->id
            );

            $this->assign('inviteLink', true);
            OW::getDocument()->addOnloadScript("
                var challengeFloatBox;
                $('#inviteLink', $('#" . $cmpId . "')).click(
                    function(){
                        challengeFloatBox = OW.ajaxFloatBox('CHALLENGE_CMP_InviteUserListSelect', " . json_encode($params) . ", {width:600, iconClass: 'ow_ic_user', title: " . json_encode(OW::getLanguage()->text('challenge', 'friends_invite_button_label')) . "});
                    }
                );
                OW.bind('base.avatar_user_list_select',
                    function(list){
                        challengeFloatBox.close();
                        $.ajax({
                            type: 'POST',
                            url: " . json_encode(OW::getRouter()->urlFor('CHALLENGE_CTRL_Base', 'inviteResponder')) . ",
                            data: 'challengeId=" . json_encode($challenge->getId()) . "&userIdList='+JSON.stringify(list),
                            dataType: 'json',
                            success : function(data){
                                if( data.messageType == 'error' ){
                                    OW.error(data.message);
                                }
                                else{
                                    OW.info(data.message);
                                }
                            },
                            error : function( XMLHttpRequest, textStatus, errorThrown ){
                                OW.error(textStatus);
                            }
                        });
                    }
                );
            ");
        }

        $cmntParams = new BASE_CommentsParams('challenge', 'challenge');
        $cmntParams->setEntityId($challenge->getId());
        $cmntParams->setOwnerId($challenge->getUserId());
        $this->addComponent('comments', new BASE_CMP_Comments($cmntParams));
        $this->addComponent('userListCmp', new CHALLENGE_CMP_ChallengeUsers($challenge->getId()));
        
        $challenge = new BASE_CLASS_ChallengeCollector(CHALLENGE_BOL_ChallengeService::CHALLENGE_COLLECT_TOOLBAR, array(
            "challengeId" => $challenge->getId()
        ));
        
        $this->assign("toolbar", $challenge->getData());
    }

    /**
     * Challenges list controller
     * 
     * @param array $params 
     */
    public function challengesList( $params )
    {
        if ( empty($params['list']) )
        {
            throw new Redirect404Exception();
        }

        $configs = $this->challengeService->getConfigs();
        $page = ( empty($_GET['page']) || (int) $_GET['page'] < 0 ) ? 1 : (int) $_GET['page'];

        $language = OW::getLanguage();

        $toolbarList = array();

        switch ( trim($params['list']) )
        {
            case 'created':
                if ( !OW::getUser()->isAuthenticated() )
                {
                    throw new Redirect403Exception();
                }

                $this->setPageHeading($language->text('challenge', 'challenge_created_by_me_page_heading'));
                $this->setPageTitle($language->text('challenge', 'challenge_created_by_me_page_title'));
                $this->setPageHeadingIconClass('ow_ic_calendar');
                $challenges = $this->challengeService->findUserChallenges(OW::getUser()->getId(), $page);
                $challengesCount = $this->challengeService->findLatestChallengesCount();
                break;

            case 'joined':
                if ( !OW::getUser()->isAuthenticated() )
                {
                    throw new Redirect403Exception();
                }
                $contentMenu = CHALLENGE_BOL_ChallengeService::getInstance()->getContentMenu();
                $this->addComponent('contentMenu', $contentMenu);
                $this->setPageHeading($language->text('challenge', 'challenge_joined_by_me_page_heading'));
                $this->setPageTitle($language->text('challenge', 'challenge_joined_by_me_page_title'));
                $this->setPageHeadingIconClass('ow_ic_calendar');

                $challenges = $this->challengeService->findUserParticipatedChallenges(OW::getUser()->getId(), $page);
                $challengesCount = $this->challengeService->findUserParticipatedChallengesCount(OW::getUser()->getId());
                break;

            case 'latest':
                $contentMenu = CHALLENGE_BOL_ChallengeService::getInstance()->getContentMenu();
                $contentMenu->getElement('latest')->setActive(true);
                $this->addComponent('contentMenu', $contentMenu);
                $this->setPageHeading($language->text('challenge', 'latest_challenges_page_heading'));
                $this->setPageTitle($language->text('challenge', 'latest_challenges_page_title'));
                $this->setPageHeadingIconClass('ow_ic_calendar');
                OW::getDocument()->setDescription($language->text('challenge', 'latest_challenges_page_desc'));
                $challenges = $this->challengeService->findPublicChallenges($page);
                $challengesCount = $this->challengeService->findPublicChallengesCount();
                break;

            case 'user-participated-challenges':

                if ( empty($_GET['userId']) )
                {
                    throw new Redirect404Exception();
                }

                $user = BOL_UserService::getInstance()->findUserById($_GET['userId']);

                if ( $user === null )
                {
                    throw new Redirect404Exception();
                }

                $challengeParams = array(
                    'action' => 'challenge_view_attend_challenges',
                    'ownerId' => $user->getId(),
                    'viewerId' => OW::getUser()->getId()
                );

                OW::getChallengeManager()->getInstance()->call('privacy_check_permission', $challengeParams);

                $displayName = BOL_UserService::getInstance()->getDisplayName($user->getId());

                $this->setPageHeading($language->text('challenge', 'user_participated_challenges_page_heading', array('display_name' => $displayName)));
                $this->setPageTitle($language->text('challenge', 'user_participated_challenges_page_title', array('display_name' => $displayName)));
                OW::getDocument()->setDescription($language->text('challenge', 'user_participated_challenges_page_desc', array('display_name' => $displayName)));
                $this->setPageHeadingIconClass('ow_ic_calendar');
                $challenges = $this->challengeService->findUserParticipatedPublicChallenges($user->getId(), $page);
                $challengesCount = $this->challengeService->findUserParticipatedPublicChallengesCount($user->getId());
                break;

            case 'past':
                $contentMenu = CHALLENGE_BOL_ChallengeService::getInstance()->getContentMenu();
                $this->addComponent('contentMenu', $contentMenu);
                $this->setPageHeading($language->text('challenge', 'past_challenges_page_heading'));
                $this->setPageTitle($language->text('challenge', 'past_challenges_page_title'));
                $this->setPageHeadingIconClass('ow_ic_calendar');
                OW::getDocument()->setDescription($language->text('challenge', 'past_challenges_page_desc'));
                $challenges = $this->challengeService->findPublicChallenges($page, null, true);
                $challengesCount = $this->challengeService->findPublicChallengesCount(true);
                break;

            case 'invited':
                if ( !OW::getUser()->isAuthenticated() )
                {
                    throw new Redirect403Exception();
                }

                $this->challengeService->hideInvitationByUserId(OW::getUser()->getId());

                $contentMenu = CHALLENGE_BOL_ChallengeService::getInstance()->getContentMenu();
                $this->addComponent('contentMenu', $contentMenu);
                $this->setPageHeading($language->text('challenge', 'invited_challenges_page_heading'));
                $this->setPageTitle($language->text('challenge', 'invited_challenges_page_title'));
                $this->setPageHeadingIconClass('ow_ic_calendar');
                $challenges = $this->challengeService->findUserInvitedChallenges(OW::getUser()->getId(), $page);
                $challengesCount = $this->challengeService->findUserInvitedChallengesCount(OW::getUser()->getId());
                
                foreach( $challenges as $challenge )
                {
                    $toolbarList[$challenge->getId()] = array();

                    $paramsList = array( 'challengeId' => $challenge->getId(), 'page' => $page, 'list' => trim($params['list']) );

                    $acceptUrl = OW::getRequest()->buildUrlQueryString(OW::getRouter()->urlForRoute('challenge.invite_accept', $paramsList), array('page' => $page));
                    $ignoreUrl = OW::getRequest()->buildUrlQueryString(OW::getRouter()->urlForRoute('challenge.invite_decline', $paramsList), array('page' => $page));

                    $toolbarList[$challenge->getId()][] = array('label' => $language->text('challenge', 'accept_request'),'href' => $acceptUrl);
                    $toolbarList[$challenge->getId()][] = array('label' => $language->text('challenge', 'ignore_request'),'href' => $ignoreUrl);
                    
                }

                break;

            default:
                throw new Redirect404Exception();
        }

        $this->addComponent('paging', new BASE_CMP_Paging($page, ceil($challengesCount / $configs[CHALLENGE_BOL_ChallengeService::CONF_CHALLENGES_COUNT_ON_PAGE]), 5));

        if ( !OW::getUser()->isAuthorized('challenge', 'add_challenge') )
        {
            $this->assign('noButton', true);
        }

        if ( empty($challenges) )
        {
            $this->assign('no_challenges', true);
        }
        
        $this->assign('listType', trim($params['list']));
        $this->assign('page', $page);
        $this->assign('challenges', $this->challengeService->getListingDataWithToolbar($challenges, $toolbarList));
        $this->assign('toolbarList', $toolbarList);
        $this->assign('add_new_url', OW::getRouter()->urlForRoute('challenge.add'));
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'challenge', 'main_menu_item');
    }

    public function inviteListAccept( $params )
    {
        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new Redirect404Exception();
        }

        $userId = OW::getUser()->getId();
        $feedback = array('messageType' => 'error');
        $exit = false;
        $attendedStatus = 1;

        if ( !empty($attendedStatus) && !empty($params['challengeId']) && $this->challengeService->canUserView($params['challengeId'], $userId) )
        {
            $challenge = $this->challengeService->findChallenge($params['challengeId']);

            if ( $challenge->getEndTimeStamp() < time() )
            {
                throw new Redirect404Exception();
            }

            $challengeUser = $this->challengeService->findChallengeUser($params['challengeId'], $userId);

            if ( $challengeUser !== null && (int) $challengeUser->getStatus() === (int) $attendedStatus )
            {
                $feedback['message'] = OW::getLanguage()->text('challenge', 'user_status_not_changed_error');
                //exit(json_encode($feedback));
            }

            if ( $challenge->getUserId() == OW::getUser()->getId() && (int) $attendedStatus == CHALLENGE_BOL_ChallengeService::USER_STATUS_NO )
            {
                $feedback['message'] = OW::getLanguage()->text('challenge', 'user_status_author_cant_leave_error');
                //exit(json_encode($feedback));
            }

            if ( !$exit )
            {
                if ( $challengeUser === null )
                {
                    $challengeUser = new CHALLENGE_BOL_ChallengeUser();
                    $challengeUser->setUserId($userId);
                    $challengeUser->setChallengeId((int) $params['challengeId']);
                }

                $challengeUser->setStatus((int) $attendedStatus);
                $challengeUser->setTimeStamp(time());
                $this->challengeService->saveChallengeUser($challengeUser);
                $this->challengeService->deleteUserChallengeInvites((int)$params['challengeId'], OW::getUser()->getId());

                $feedback['message'] = OW::getLanguage()->text('challenge', 'user_status_updated');
                $feedback['messageType'] = 'info';

                if ( $challengeUser->getStatus() == CHALLENGE_BOL_ChallengeService::USER_STATUS_YES && $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_ANYBODY )
                {
                    $userName = BOL_UserService::getInstance()->getDisplayName($challenge->getUserId());
                    $userUrl = BOL_UserService::getInstance()->getUserUrl($challenge->getUserId());
                    $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

                    OW::getChallengeManager()->trigger(new OW_Challenge('feed.activity', array(
                            'activityType' => 'challenge-join',
                            'activityId' => $challengeUser->getId(),
                            'entityId' => $challenge->getId(),
                            'entityType' => 'challenge',
                            'userId' => $challengeUser->getUserId(),
                            'pluginKey' => 'challenge'
                            ), array(
                            'challengeId' => $challenge->getId(),
                            'userId' => $challengeUser->getUserId(),
                            'challengeUserId' => $challengeUser->getId(),
                            'string' =>  OW::getLanguage()->text('challenge', 'feed_actiovity_attend_string' ,  array( 'user' => $userEmbed )),
                            'feature' => array()
                        )));
                }
            }
        }
        else
        {
            $feedback['message'] = OW::getLanguage()->text('challenge', 'user_status_update_error');
        }

        if ( !empty($feedback['message']) )
        {
            switch( $feedback['messageType'] )
            {
                case 'info':
                    OW::getFeedback()->info($feedback['message']);
                    break;
                case 'warning':
                    OW::getFeedback()->warning($feedback['message']);
                    break;
                case 'error':
                    OW::getFeedback()->error($feedback['message']);
                    break;
            }
        }

        $paramsList = array();

        if ( !empty($params['page']) )
        {
            $paramsList['page'] = $params['page'];
        }

        if ( !empty($params['list']) )
        {
            $paramsList['list'] = $params['list'];
        }

        $this->redirect(OW::getRouter()->urlForRoute('challenge.view_challenge_list', $paramsList));
    }

    public function inviteListDecline( $params )
    {
        if ( !empty($params['challengeId']) )
        {
            $this->challengeService->deleteUserChallengeInvites((int)$params['challengeId'], OW::getUser()->getId());
            OW::getLanguage()->text('challenge', 'user_status_updated');
        }
        else
        {
            OW::getLanguage()->text('challenge', 'user_status_update_error');
        }

        if ( !empty($params['page']) )
        {
            $paramsList['page'] = $params['page'];
        }

        if ( !empty($params['list']) )
        {
            $paramsList['list'] = $params['list'];
        }

        $this->redirect(OW::getRouter()->urlForRoute('challenge.view_challenge_list', $paramsList));
    }

    /**
     * User's challenges list controller
     * 
     * @param array $params 
     */
    public function challengeUserLists( $params )
    {
        if ( empty($params['challengeId']) || empty($params['list']) )
        {
            throw new Redirect404Exception();
        }

        $challenge = $this->challengeService->findChallenge((int) $params['challengeId']);

        if ( $challenge === null )
        {
            throw new Redirect404Exception();
        }

        $listArray = array_flip($this->challengeService->getUserListsArray());

        if ( !array_key_exists($params['list'], $listArray) )
        {
            throw new Redirect404Exception();
        }

        if ( !OW::getUser()->isAuthorized('challenge', 'view_challenge') && $challenge->getUserId() != OW::getUser()->getId() && !OW::getUser()->isAuthorized('challenge') )
        {
            $this->assign('authErrorText', OW::getLanguage()->text('challenge', 'challenge_view_permission_error_message'));
            return;
        }

        // guest gan't view private challenges
        if ( (int) $challenge->getWhoCanView() === CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY && !OW::getUser()->isAuthenticated() )
        {
            $this->redirect(OW::getRouter()->urlForRoute('challenge.private_challenge', array('challengeId' => $challenge->getId())));
        }

        $challengeInvite = $this->challengeService->findChallengeInvite($challenge->getId(), OW::getUser()->getId());
        $challengeUser = $this->challengeService->findChallengeUser($challenge->getId(), OW::getUser()->getId());

        // check if user can view challenge
        if ( (int) $challenge->getWhoCanView() === CHALLENGE_BOL_ChallengeService::CAN_VIEW_INVITATION_ONLY && $challengeUser === null && $challengeInvite === null && !OW::getUser()->isAuthorized('challenge') )
        {
            $this->redirect(OW::getRouter()->urlForRoute('challenge.private_challenge', array('challengeId' => $challenge->getId())));
        }

        $language = OW::getLanguage();
        $configs = $this->challengeService->getConfigs();
        $page = ( empty($_GET['page']) || (int) $_GET['page'] < 0 ) ? 1 : (int) $_GET['page'];
        $status = $listArray[$params['list']];
        $challengeUsers = $this->challengeService->findChallengeUsers($challenge->getId(), $status, $page);
        $challengeUsersCount = $this->challengeService->findChallengeUsersCount($challenge->getId(), $status);

        $userIdList = array();

        /* @var $challengeUser CHALLENGE_BOL_ChallengeUser */
        foreach ( $challengeUsers as $challengeUser )
        {
            $userIdList[] = $challengeUser->getUserId();
        }

        $userDtoList = BOL_UserService::getInstance()->findUserListByIdList($userIdList);

        $this->addComponent('users', new CHALLENGE_CMP_ChallengeUsersList($userDtoList, $challengeUsersCount, $configs[CHALLENGE_BOL_ChallengeService::CONF_CHALLENGE_USERS_COUNT_ON_PAGE], true));

        $this->setPageHeading($language->text('challenge', 'user_list_page_heading_' . $status, array('challengeTitle' => $challenge->getTitle())));
        $this->setPageTitle($language->text('challenge', 'user_list_page_heading_' . $status, array('challengeTitle' => $challenge->getTitle())));
        OW::getDocument()->setDescription($language->text('challenge', 'user_list_page_desc_' . $status, array('challengeTitle' => $challenge->getTitle())));

        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'challenge', 'main_menu_item');
    }

    public function privateChallenge( $params )
    {
        $language = OW::getLanguage();

        $this->setPageTitle($language->text('challenge', 'private_page_title'));
        $this->setPageHeading($language->text('challenge', 'private_page_heading'));
        $this->setPageHeadingIconClass('ow_ic_lock');

        $challengeId = $params['challengeId'];
        $challenge = $this->challengeService->findChallenge((int) $challengeId);

        $avatarList = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($challenge->userId));
        $displayName = BOL_UserService::getInstance()->getDisplayName($challenge->userId);
        $userUrl = BOL_UserService::getInstance()->getUserUrl($challenge->userId);

        $this->assign('challenge', $challenge);
        $this->assign('avatar', $avatarList[$challenge->userId]);
        $this->assign('displayName', $displayName);
        $this->assign('userUrl', $userUrl);
        $this->assign('creator', $language->text('challenge', 'creator'));
    }
    
    /**
     * Responder for challenge attend form
     */
    public function attendFormResponder()
    {
        if ( !OW::getRequest()->isAjax() || !OW::getUser()->isAuthenticated() )
        {
            throw new Redirect404Exception();
        }

        $userId = OW::getUser()->getId();
        $respondArray = array('messageType' => 'error');
        
        if ( !empty($_POST['attend_status']) && in_array((int) $_POST['attend_status'], array(1, 2, 3)) && !empty($_POST['challengeId']) && $this->challengeService->canUserView($_POST['challengeId'], $userId) )
        {
            $challenge = $this->challengeService->findChallenge($_POST['challengeId']);
            
            if ( $challenge->getEndTimeStamp() < time() )
            {
                throw new Redirect404Exception();
            }

            $challengeUser = $this->challengeService->findChallengeUser($_POST['challengeId'], $userId);

            if ( $challengeUser !== null && (int) $challengeUser->getStatus() === (int) $_POST['attend_status'] )
            {
                $respondArray['message'] = OW::getLanguage()->text('challenge', 'user_status_not_changed_error');
                exit(json_encode($respondArray));
            }

            if ( $challenge->getUserId() == OW::getUser()->getId() && (int) $_POST['attend_status'] == CHALLENGE_BOL_ChallengeService::USER_STATUS_NO )
            {
                $respondArray['message'] = OW::getLanguage()->text('challenge', 'user_status_author_cant_leave_error');
                exit(json_encode($respondArray));
            }

            if ( $challengeUser === null )
            {
                $challengeUser = new CHALLENGE_BOL_ChallengeUser();
                $challengeUser->setUserId($userId);
                $challengeUser->setChallengeId((int) $_POST['challengeId']);
            }

            $challengeUser->setStatus((int) $_POST['attend_status']);
            $challengeUser->setTimeStamp(time());
            $this->challengeService->saveChallengeUser($challengeUser);

            $this->challengeService->deleteUserChallengeInvites((int)$_POST['challengeId'], OW::getUser()->getId());

            $e = new OW_Challenge(CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CHANGE_USER_STATUS, array('challengeId' => $challenge->id, 'userId' => $challengeUser->userId));
            OW::getChallengeManager()->trigger($e);
            
            $respondArray['message'] = OW::getLanguage()->text('challenge', 'user_status_updated');
            $respondArray['messageType'] = 'info';
            $respondArray['currentLabel'] = OW::getLanguage()->text('challenge', 'user_status_label_' . $challengeUser->getStatus());
            $respondArray['challengeId'] = (int) $_POST['challengeId'];
            //$challengeUsersCmp = new CHALLENGE_CMP_ChallengeUsers((int) $_POST['challengeId']);
            //$respondArray['challengeUsersCmp'] = $challengeUsersCmp->render();
            $respondArray['newInvCount'] = $this->challengeService->findUserInvitedChallengesCount(OW::getUser()->getId());

            if ( $challengeUser->getStatus() == CHALLENGE_BOL_ChallengeService::USER_STATUS_YES && $challenge->getWhoCanView() == CHALLENGE_BOL_ChallengeService::CAN_VIEW_ANYBODY )
            {
//                $params = array(
//                    'pluginKey' => 'challenge',
//                    'entityType' => 'challenge_join',
//                    'entityId' => $challengeUser->getUserId(),
//                    'userId' => $challengeUser->getUserId()
//                );
//
//                $url = OW::getRouter()->urlForRoute('challenge.view', array('challengeId' => $challenge->getId()));
//                $thumb = $this->challengeService->generateImageUrl($challenge->getId(), true);
//
//
//
//                $dataValue = array(
//                    'time' => $challengeUser->getTimeStamp(),
//                    'string' => OW::getLanguage()->text('challenge', 'feed_user_join_string'),
//                    'content' => '<div class="clearfix"><div class="ow_newsfeed_item_picture">
//                        <a href="' . $url . '"><img src="' . $thumb . '" /></a>
//                        </div><div class="ow_newsfeed_item_content">
//                        <a class="ow_newsfeed_item_title" href="' . $url . '">' . $challenge->getTitle() . '</a><div class="ow_remark">' . strip_tags($challenge->getDescription()) . '</div></div></div>',
//                    'view' => array(
//                        'iconClass' => 'ow_ic_calendar'
//                    )
//                );
//
//                $fChallenge = new OW_Challenge('feed.action', $params, $dataValue);
//                OW::getChallengeManager()->trigger($fChallenge);

                $userName = BOL_UserService::getInstance()->getDisplayName($challenge->getUserId());
                $userUrl = BOL_UserService::getInstance()->getUserUrl($challenge->getUserId());
                $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

                OW::getChallengeManager()->trigger(new OW_Challenge('feed.activity', array(
                        'activityType' => 'challenge-join',
                        'activityId' => $challengeUser->getId(),
                        'entityId' => $challenge->getId(),
                        'entityType' => 'challenge',
                        'userId' => $challengeUser->getUserId(),
                        'pluginKey' => 'challenge'
                        ), array(
                        'challengeId' => $challenge->getId(),
                        'userId' => $challengeUser->getUserId(),
                        'challengeUserId' => $challengeUser->getId(),
                        'string' =>  OW::getLanguage()->text('challenge', 'feed_actiovity_attend_string' ,  array( 'user' => $userEmbed )),
                        'feature' => array()
                    )));
            }
        }
        else
        {
            $respondArray['message'] = OW::getLanguage()->text('challenge', 'user_status_update_error');
        }

        exit(json_encode($respondArray));
    }

    /**
     * Responder for challenge invite form
     */
    public function inviteResponder()
    {
        $respondArray = array();

        if ( empty($_POST['challengeId']) || empty($_POST['userIdList']) || !OW::getUser()->isAuthenticated() )
        {
            $respondArray['messageType'] = 'error';
            $respondArray['message'] = '_ERROR_';
            echo json_encode($respondArray);
            exit;
        }

        $idList = json_decode($_POST['userIdList']);

        if ( empty($_POST['challengeId']) || empty($idList) )
        {
            $respondArray['messageType'] = 'error';
            $respondArray['message'] = '_EMPTY_CHALLENGE_ID_';
            echo json_encode($respondArray);
            exit;
        }

        $challenge = $this->challengeService->findChallenge($_POST['challengeId']);

        if ( $challenge->getEndTimeStamp() < time() )
        {
            throw new Redirect404Exception();
        }

        if ( $challenge === null )
        {
            $respondArray['messageType'] = 'error';
            $respondArray['message'] = '_EMPTY_CHALLENGE_';
            echo json_encode($respondArray);
            exit;
        }

        if ( (int) $challenge->getUserId() === OW::getUser()->getId() || (int) $challenge->getWhoCanInvite() === CHALLENGE_BOL_ChallengeService::CAN_INVITE_PARTICIPANT )
        {
            $count = 0;

            $userList = BOL_UserService::getInstance()->findUserListByIdList($idList);

            foreach ( $userList as $user )
            {
                $userId = $user->id;
                $challengeInvite = $this->challengeService->findChallengeInvite($challenge->getId(), $userId);

                if ( $challengeInvite === null )
                {
                    $challengeInvite = $this->challengeService->inviteUser($challenge->getId(), $userId, OW::getUser()->getId());
                    $challengeObj = new OW_Challenge('challenge.invite_user', array('userId' => $userId, 'inviterId' => OW::getUser()->getId(), 'challengeId' => $challenge->getId(), 'imageId' => $challenge->getImage(), 'challengeTitle' => $challenge->getTitle(), 'challengeDesc' => $challenge->getDescription(), 'displayInvitation' => $challengeInvite->displayInvitation));
                    OW::getChallengeManager()->trigger($challengeObj);
                    $count++;
                }
            }
        }

        $respondArray['messageType'] = 'info';
        $respondArray['message'] = OW::getLanguage()->text('challenge', 'users_invite_success_message', array('count' => $count));

        exit(json_encode($respondArray));
    }
}

/**
 * Challenge attend form
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_plugins.challenge.forms
 * @since 1.0
 */
class AttendForm extends Form
{

    public function __construct( $challengeId, $contId )
    {
        parent::__construct('challenge_attend');
        $this->setAction(OW::getRouter()->urlFor('CHALLENGE_CTRL_Base', 'attendFormResponder'));
        $this->setAjax();
        $hidden = new HiddenField('attend_status');
        $this->addElement($hidden);
        $challengeIdField = new HiddenField('challengeId');
        $challengeIdField->setValue($challengeId);
        $this->addElement($challengeIdField);
        $this->setAjaxResetOnSuccess(false);
        $this->bindJsFunction(Form::BIND_SUCCESS, "function(data){
            var \$context = $('#" . $contId . "');

            

            if(data.messageType == 'error'){
                OW.error(data.message);
            }
            else{
                $('.current_status span.status', \$context).empty().html(data.currentLabel);
                $('.current_status span.link', \$context).css({display:'inline'});
                $('.attend_buttons .buttons', \$context).fadeOut(500);

                if ( data.challengeId != 'undefuned' )
                {
                    OW.loadComponent('CHALLENGE_CMP_ChallengeUsers', {challengeId: data.challengeId},
                    {
                      onReady: function( html ){
                         $('.userList', \$context).empty().html(html);

                      }
                    });
                }

                $('.userList', \$context).empty().html(data.challengeUsersCmp);
                OW.trigger('challenge_notifications_update', {count:data.newInvCount});
                OW.info(data.message);
            }
        }");
    }
}


/**
 * Add new challenge form
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_plugins.challenge.forms
 * @since 1.0
 */
class ChallengeAddForm extends Form
{

    const CHALLENGE_NAME = 'challenge.challenge_add_form.get_element';

    public function __construct( $name )
    {
        parent::__construct($name);

        $militaryTime = Ow::getConfig()->getValue('base', 'military_time');

        $language = OW::getLanguage();

        $currentYear = date('Y', time());

        $title = new TextField('title');
        $title->setRequired();
        $title->setLabel($language->text('challenge', 'add_form_title_label'));

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'title' ), $title);
        OW::getChallengeManager()->trigger($challenge);
        $title = $challenge->getData();

        $this->addElement($title);

        $startDate = new DateField('start_date');
        $startDate->setMinYear($currentYear);
        $startDate->setMaxYear($currentYear + 5);
        $startDate->setRequired();

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'start_date' ), $startDate);
        OW::getChallengeManager()->trigger($challenge);
        $startDate = $challenge->getData();

        $this->addElement($startDate);

        $startTime = new ChallengeTimeField('start_time');
        $startTime->setMilitaryTime($militaryTime);
        
        if ( !empty($_POST['endDateFlag']) )
        {
            $startTime->setRequired();
        }

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'start_time' ), $startTime);
        OW::getChallengeManager()->trigger($challenge);
        $startTime = $challenge->getData();

        $this->addElement($startTime);

        $endDate = new DateField('end_date');
        $endDate->setMinYear($currentYear);
        $endDate->setMaxYear($currentYear + 5);

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'end_date' ), $endDate);
        OW::getChallengeManager()->trigger($challenge);
        $endDate = $challenge->getData();

        $this->addElement($endDate);

        $endTime = new ChallengeTimeField('end_time');
        $endTime->setMilitaryTime($militaryTime);

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'end_time' ), $endTime);
        OW::getChallengeManager()->trigger($challenge);
        $endTime = $challenge->getData();

        $this->addElement($endTime);

        $location = new TextField('location');
        $location->setRequired();
        $location->setLabel($language->text('challenge', 'add_form_location_label'));

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'location' ), $location);
        OW::getChallengeManager()->trigger($challenge);
        $location = $challenge->getData();

        $this->addElement($location);

        $whoCanView = new RadioField('who_can_view');
        $whoCanView->setRequired();
        $whoCanView->addOptions(
            array(
                '1' => $language->text('challenge', 'add_form_who_can_view_option_anybody'),
                '2' => $language->text('challenge', 'add_form_who_can_view_option_invit_only')
            )
        );
        $whoCanView->setLabel($language->text('challenge', 'add_form_who_can_view_label'));

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'who_can_view' ), $whoCanView);
        OW::getChallengeManager()->trigger($challenge);
        $whoCanView = $challenge->getData();

        $this->addElement($whoCanView);

        $whoCanInvite = new RadioField('who_can_invite');
        $whoCanInvite->setRequired();
        $whoCanInvite->addOptions(
            array(
                CHALLENGE_BOL_ChallengeService::CAN_INVITE_PARTICIPANT => $language->text('challenge', 'add_form_who_can_invite_option_participants'),
                CHALLENGE_BOL_ChallengeService::CAN_INVITE_CREATOR => $language->text('challenge', 'add_form_who_can_invite_option_creator')
            )
        );
        $whoCanInvite->setLabel($language->text('challenge', 'add_form_who_can_invite_label'));

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'who_can_invite' ), $whoCanInvite);
        OW::getChallengeManager()->trigger($challenge);
        $whoCanInvite = $challenge->getData();

        $this->addElement($whoCanInvite);
		
		$Status = new RadioField('status');
        $Status->addOptions(
            array(
                '1' => $language->text('challenge', 'running'),
                '2' => $language->text('challenge', 'completed'),
				'3' => $language->text('challenge', 'failed')
            )
        );
        $Status->setLabel('Estado');

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'status' ), $Status);
        OW::getChallengeManager()->trigger($challenge);
        $Status = $challenge->getData();

        $this->addElement($Status);

        $submit = new Submit('submit');
        $submit->setValue($language->text('challenge', 'add_form_submit_label'));
        $this->addElement($submit);

        $desc = new WysiwygTextarea('desc');
        $desc->setLabel($language->text('challenge', 'add_form_desc_label'));
        $desc->setRequired();

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'desc' ), $desc);
        OW::getChallengeManager()->trigger($challenge);
        $desc = $challenge->getData();

        $this->addElement($desc);

        $imageField = new FileField('image');
        $imageField->setLabel($language->text('challenge', 'add_form_image_label'));
        $this->addElement($imageField);

        $challenge = new OW_Challenge(self::CHALLENGE_NAME, array( 'name' => 'image' ), $imageField);
        OW::getChallengeManager()->trigger($challenge);
        $imageField = $challenge->getData();

        $this->setEnctype(Form::ENCTYPE_MULTYPART_FORMDATA);
    }
}

/**
 * Form element: CheckboxField.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_core
 * @since 1.0
 */
class ChallengeTimeField extends FormElement
{
    private $militaryTime;

    private $allDay = false;

    /**
     * Constructor.
     *
     * @param string $name
     */
    public function __construct( $name )
    {
        parent::__construct($name);
        $this->militaryTime = false;
    }

    public function setMilitaryTime( $militaryTime )
    {
        $this->militaryTime = (bool) $militaryTime;
    }

    public function setValue( $value )
    {
        if ( $value === null )
        {
            $this->value = null;
        }

        $this->allDay = false;
        
        if ( $value === 'all_day' )
        {
            $this->allDay = true;
            $this->value = null;
            return;
        }

        if ( is_array($value) && isset($value['hour']) && isset($value['minute']) )
        {
            $this->value = array_map('intval', $value);
        }

        if ( is_string($value) && strstr($value, ':') )
        {
            $parts = explode(':', $value);
            $this->value['hour'] = (int) $parts[0];
            $this->value['minute'] = (int) $parts[1];
        }
    }

    public function getValue()
    {
        if ( $this->allDay === true )
        {
            return 'all_day';
        }

        return $this->value;
    }

    /**
     *
     * @return string
     */
    public function getElementJs()
    {
        $jsString = "var formElement = new OwFormElement('" . $this->getId() . "', '" . $this->getName() . "');";

        /** @var $value Validator  */
        foreach ( $this->validators as $value )
        {
            $jsString .= "formElement.addValidator(" . $value->getJsValidator() . ");";
        }

        return $jsString;
    }

    private function getTimeString( $hour, $minute )
    {
        if ( $this->militaryTime )
        {
            $hour = $hour < 10 ? '0' . $hour : $hour;
            return $hour . ':' . $minute;
        }
        else
        {
            if ( $hour == 12 )
            {
                $dp = 'pm';
            }
            else if ( $hour > 12 )
            {
                $hour = $hour - 12;
                $dp = 'pm';
            }
            else
            {
                $dp = 'am';
            }

            $hour = $hour < 10 ? '0' . $hour : $hour;
            return $hour . ':' . $minute . $dp;
        }
    }

    /**
     * @see FormElement::renderInput()
     *
     * @param array $params
     * @return string
     */
    public function renderInput( $params = null )
    {
        parent::renderInput($params);
        
        for ( $hour = 0; $hour <= 23; $hour++ )
        {
            $valuesArray[$hour . ':0'] = array('label' => $this->getTimeString($hour, '00'), 'hour' => $hour, 'minute' => 0);
            $valuesArray[$hour . ':30'] = array('label' => $this->getTimeString($hour, '30'), 'hour' => $hour, 'minute' => 30);
        }

        $optionsString = UTIL_HtmlTag::generateTag('option', array('value' => ""), true, OW::getLanguage()->text('challenge', 'time_field_invitation_label'));

        $allDayAttrs = array( 'value' => "all_day"  );
        
        if ( $this->allDay )
        {
            $allDayAttrs['selected'] = 'selected';
        }
        
        $optionsString = UTIL_HtmlTag::generateTag('option', $allDayAttrs, true, OW::getLanguage()->text('challenge', 'all_day'));

        foreach ( $valuesArray as $value => $labelArr )
        {
            $attrs = array('value' => $value);

            if ( !empty($this->value) && $this->value['hour'] === $labelArr['hour'] && $this->value['minute'] === $labelArr['minute'] )
            {
                $attrs['selected'] = 'selected';
            }

            $optionsString .= UTIL_HtmlTag::generateTag('option', $attrs, true, $labelArr['label']);
        } 

        return UTIL_HtmlTag::generateTag('select', $this->attributes, true, $optionsString);
    }
}

class CHALLENGE_CMP_ChallengeUsersList extends BASE_CMP_Users
{

    public function getFields( $userIdList )
    {
        $fields = array();

        $qs = array();

        $qBdate = BOL_QuestionService::getInstance()->findQuestionByName('birthdate', 'sex');

        if ( $qBdate->onView )
            $qs[] = 'birthdate';

        $qSex = BOL_QuestionService::getInstance()->findQuestionByName('sex');

        if ( $qSex->onView )
            $qs[] = 'sex';

        $questionList = BOL_QuestionService::getInstance()->getQuestionData($userIdList, $qs);

        foreach ( $questionList as $uid => $q )
        {

            $fields[$uid] = array();

            $age = '';

            if ( !empty($q['birthdate']) )
            {
                $date = UTIL_DateTime::parseDate($q['birthdate'], UTIL_DateTime::MYSQL_DATETIME_DATE_FORMAT);

                $age = UTIL_DateTime::getAge($date['year'], $date['month'], $date['day']);
            }

            if ( !empty($q['sex']) )
            {
                $fields[$uid][] = array(
                    'label' => '',
                    'value' => BOL_QuestionService::getInstance()->getQuestionValueLang('sex', $q['sex']) . ' ' . $age
                );
            }

            if ( !empty($q['birthdate']) )
            {
                $dinfo = date_parse($q['birthdate']);
            }
        }

        return $fields;
    }
}
