<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Oxwall Foundation (http://www.oxwall.org/foundation).
 * All portions of the code written by Oxwall Foundation are Copyright (c) 2011. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2011 Oxwall Foundation. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * @author Egor Bulgakov <egor.bulgakov@gmail.com>
 * @package ow.ow_plugins.challenge.mobile.classes
 * @since 1.6.0
 */
class CHALLENGE_MCLASS_ChallengeHandler
{
    /**
     * Class instance
     *
     * @var CHALLENGE_MCLASS_ChallengeHandler
     */
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return CHALLENGE_MCLASS_ChallengeHandler
     */
    public static function getInstance()
    {
        if ( !isset(self::$classInstance) )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    public function setInvitationData( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();
        if ( $params['entityType'] == 'challenge-join' )
        {
            $data = $params['data'];
            $data['string']['vars']['challenge'] = strip_tags($data['string']['vars']['challenge']);
            $data['acceptCommand'] = 'challenges.accept';
            $data['declineCommand'] = 'challenges.ignore';
            $challenge->setData($data);
        }
    }

    public function onCommand( OW_Challenge $challenge )
    {
        if ( !OW::getUser()->isAuthenticated() )
        {
            return;
        }

        $params = $challenge->getParams();

        if ( !in_array($params['command'], array('challenges.accept', 'challenges.ignore')) )
        {
            return;
        }

        $challengeId = $params['data'];
        $challengeDto = CHALLENGE_BOL_ChallengeService::getInstance()->findChallenge($challengeId);

        $userId = OW::getUser()->getId();
        $challengeService = CHALLENGE_BOL_ChallengeService::getInstance();

        if ( empty($challengeDto) )
        {
            BOL_InvitationService::getInstance()->deleteInvitation(
                CHALLENGE_CLASS_InvitationHandler::INVITATION_JOIN, $challengeId, $userId
            );

            return;
        }

        $lang = OW::getLanguage();
        $result = array('result' => false);

        if ( $params['command'] == 'challenges.accept' )
        {
            $exit = false;
            $attendedStatus = 1;

            if ( $challengeService->canUserView($challengeId, $userId) )
            {
                $challengeDto = $challengeService->findChallenge($challengeId);

                if ( $challengeDto->getEndTimeStamp() < time() )
                {
                    $challengeService->deleteUserChallengeInvites((int)$challengeId, $userId);
                    $result['msg'] = $lang->text('challenge', 'user_status_updated');
                    $challenge->setData($result);

                    return;
                }

                $challengeUser = $challengeService->findChallengeUser($challengeId, $userId);

                if ( $challengeUser !== null && (int) $challengeUser->getStatus() === (int) $attendedStatus )
                {
                    $result['msg'] = $lang->text('challenge', 'user_status_not_changed_error');
                    $exit = true;
                }

                if ( $challengeDto->getUserId() == OW::getUser()->getId() && (int) $attendedStatus == CHALLENGE_BOL_ChallengeService::USER_STATUS_NO )
                {
                    $result['msg'] = $lang->text('challenge', 'user_status_author_cant_leave_error');
                    $exit = true;
                }

                if ( !$exit )
                {
                    $challengeUserDto = CHALLENGE_BOL_ChallengeService::getInstance()->addChallengeUser($userId, $challengeId, $attendedStatus);

                    if ( !empty( $challengeUserDto ) )
                    {
                        $e = new OW_Challenge(
                            CHALLENGE_BOL_ChallengeService::CHALLENGE_ON_CHANGE_USER_STATUS,
                            array('challengeId' => $challengeDto->id, 'userId' => $challengeUserDto->userId)
                        );
                        OW::getChallengeManager()->trigger($e);

                        $result = array('result' => true, 'msg' => $lang->text('challenge', 'user_status_updated'));
                        BOL_InvitationService::getInstance()->deleteInvitation(
                            CHALLENGE_CLASS_InvitationHandler::INVITATION_JOIN, $challengeId, $userId
                        );
                    }
                    else
                    {
                        $result['msg'] = $lang->text('challenge', 'user_status_update_error');
                    }
                }
            }
            else
            {
                $result['msg'] = $lang->text('challenge', 'user_status_update_error');
            }
        }
        else if ( $params['command'] == 'challenges.ignore' )
        {
            $challengeService->deleteUserChallengeInvites((int)$challengeId, $userId);
            $result = array('result' => true, 'msg' => $lang->text('challenge', 'user_status_updated'));
            BOL_InvitationService::getInstance()->deleteInvitation(
                CHALLENGE_CLASS_InvitationHandler::INVITATION_JOIN, $challengeId, $userId
            );
        }

        $challenge->setData($result);
    }

    public function init()
    {
        $em = OW::getChallengeManager();
        $em->bind('mobile.invitations.on_item_render', array($this, 'setInvitationData'));
        $em->bind('invitations.on_command', array($this, 'onCommand'));
        $em->bind('feed.on_item_render', array($this, 'onFeedItemRenderDisableActions'));
    }

    public function onFeedItemRenderDisableActions( OW_Challenge $challenge )
    {
        $params = $challenge->getParams();

        if ( $params["action"]["entityType"] != 'challenge' )
        {
            return;
        }

        $data = $challenge->getData();

        $data["disabled"] = true;

        $challenge->setData($data);
    }
}